<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2019
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_autogrid
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\AutoGrid\Backend;

/**
 * Class file
 * BackendIntegration
 */
class BackendIntegration extends \Contao\Backend
{
	/**
	 * Include grid css and jquery on backend template
	 * called from initializeSystem Hook
	 */
	public function loadAssets()
	{
		if( TL_MODE != 'BE' )
		{
			return;
		}
			
		if(in_array(\Contao\Input::get('act'), array('edit','show')) || (\Contao\Input::get('field') != '' || \Contao\Input::get('name') != '' || \Contao\Input::get('switch') != '') || \Contao\Environment::get('isAjaxRequest') )
		{
			return;
		}
		else if( (boolean)\Contao\Config::get('pct_autogrid_disable_be_preview') === true || (boolean)$GLOBALS['PCT_AUTOGRID']['assetsLoaded'] === true )
		{
			// flag
			$GLOBALS['PCT_AUTOGRID']['assetsLoaded'] = true;
			return;
		}
			
		// load backend css, grid_presets.css
		$GLOBALS['TL_CSS'][] = $GLOBALS['PCT_AUTOGRID']['css'];
		$GLOBALS['TL_CSS'][] = $GLOBALS['PCT_AUTOGRID']['be_css'];
		$GLOBALS['TL_CSS'][] = $GLOBALS['PCT_AUTOGRID']['presets_css'];
		
		// load jquery
		$strJquery = '';
		if(version_compare(VERSION, '3.5','<='))
		{
			$strJquery = TL_ASSETS_URL.'/assets/jquery/core/'.$GLOBALS['TL_ASSETS']['JQUERY'].'/jquery.min.js';
		}
		else
		{
			$strJquery = TL_ASSETS_URL.'assets/jquery/js/jquery.min.js';
		}
		// load from CDN
		if(file_exists(TL_ROOT.'/'.$strJquery) === false)
		{
			$strJquery = 'https://code.jquery.com/jquery-3.3.1.min.js';
		}
		$GLOBALS['TL_JAVASCRIPT'][] = $strJquery;
		
		$GLOBALS['TL_HEAD::PCT_AUTOGRID'][] = '<script>jQuery.noConflict();</script>';
		
		// flag
		$GLOBALS['PCT_AUTOGRID']['assetsLoaded'] = true;
	}


	/**
	 * Load Autogrid scripts to backend template
	 * @object template
	 */
	public function parseTemplateCallback($objTemplate)
	{
		if( TL_MODE != 'BE' )
		{
			return;
		}
		
		if( in_array(\Contao\Input::get('act'), array('edit','show')) || \Contao\Input::get('switch') != '' || \Contao\Environment::get('isAjaxRequest') )
		{
			return;
		}
		else if( (boolean)\Contao\Config::get('pct_autogrid_disable_be_preview') === true )
		{
			return;
		}
		
		if($objTemplate->getName() == 'be_main')
		{
			// <head>
			#$objTemplate->javascripts .= '<script>jQuery.noConflict();</script>';
			
			// pass Autogrid related head content TL_HEAD 
			if( is_array($GLOBALS['TL_HEAD::PCT_AUTOGRID']) && empty($GLOBALS['TL_HEAD::PCT_AUTOGRID']) === false)
			{
				$objTemplate->javascripts .= implode('',$GLOBALS['TL_HEAD::PCT_AUTOGRID']);
			
				// @var object
				$objAutoGridTpl = new \Contao\BackendTemplate('be_js_autogrid');
				$objAutoGridTpl->context = 'body';
				
				// </body>
				$objTemplate->mootools .= $objAutoGridTpl->parse();
			}
		}
	}


	/**
	 * Apply the backend 100% settings
	 * @param object
	 */
	public function backend100($objTemplate)
	{
		if(TL_MODE != 'BE' || $objTemplate->getName() != 'be_main')
		{
			return;
		}

		$objTemplate->ua .= ' contao'.str_replace('.','-',VERSION);

		$this->import('BackendUser','User');
		
		if( (boolean)$this->User->backend100 === true )
		{
			$GLOBALS['TL_CSS'][] = $GLOBALS['PCT_AUTOGRID']['be_css'];
			$objTemplate->ua .= ' backend100';
		}
	}
}