<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2014
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_autogrid
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\AutoGrid\Backend;

/**
 * Imports
 */
use PCT\AutoGrid\Models\ContentModel as ContentModel;
use PCT\AutoGrid\Core as AutoGrid;

/**
 * Class file
 * TableContent
 */
class TableContent extends \Contao\Backend
{
	/**
	 * Table
	 */
	protected $strTable = 'tl_content';
	
	
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser', 'User');
		
		$this->Session = \Contao\Session::getInstance();
			
		if( version_compare(VERSION, '4.4', '>=') )
		{
			$this->Session = \Contao\System::getContainer()->get('session');
			#$this->Session = \Contao\System::getContainer()->get('session')->getBag('contao_backend');
		}
		
		if( $this->Database === null )
		{
			$this->Database = \Contao\Database::getInstance();
		}
	}


	/**
	 * Modify the DCA
	 * @param object
	 * 
	 * called from onload_callback
	 */
	public function modifyDCA($objDC)
	{
		if( $objDC->activeRecord === null )
		{
			$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		}

		// load specific templates
		if( \version_compare(VERSION,'4.9','>=') && \in_array($objDC->activeRecord->type, $GLOBALS['PCT_AUTOGRID']['wrapperElements']) )
		{
			$GLOBALS['TL_DCA'][$objDC->table]['fields']['customTpl']['options_callback'] = array(\get_class($this),'loadTemplates');
		}

		// margin fields
		foreach ($GLOBALS['TL_DCA'][$objDC->table]['palettes'] as $k => $v) 
		{
			if( $k == '__selector__' )
			{
				continue;
			}
			// inject after cssID field
			$GLOBALS['TL_DCA'][$objDC->table]['palettes'][$k] = str_replace('cssID', 'cssID,margin_t,margin_b,margin_t_m,margin_b_m,', $GLOBALS['TL_DCA'][$objDC->table]['palettes'][$k]);
		}

		// autogrid fields
		$strPalette = $GLOBALS['TL_DCA'][$objDC->table]['palettes'][$objDC->activeRecord->type];

		// make sure the palette has not been modified before
		if( strlen(strpos($strPalette, ',autogrid')) < 1 )
		{
			$replace = ',protected';
			if( version_compare(VERSION, '3.5','<=') )
			{
				$replace = ',space';
			}

			if(strlen(strpos($strPalette, $replace)) > 0)
			{
				$strPalette = str_replace($replace,$replace.';{autogrid_legend},autogrid;',$strPalette);
			}
			else
			{
				$strPalette .= ';{autogrid_legend},autogrid;';
			}
			$GLOBALS['TL_DCA'][$objDC->table]['palettes'][$objDC->activeRecord->type] = $strPalette;
		}
	}


	/**
	 * Load templates to customTpl field
	 * @param object
	 * @return array
	 */
	public function loadTemplates($objDC)
	{
		if( $objDC->activeRecord === null )
		{
			$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		}

		$arrReturn = array();

		switch( $objDC->activeRecord->type )
		{
			case 'autogridRowStart':
			case 'autogridRowStop':
				$arrReturn = $this->getTemplateGroup('ce_autogrid_row',array(),'ce_autogrid_row');
				unset( $arrReturn['ce_autogrid_row'] );
				break;
			case 'autogridGridStart':
			case 'autogridGridStop':
				$arrReturn = $this->getTemplateGroup('ce_autogrid_grid',array(),'ce_autogrid_grid');
				unset( $arrReturn['ce_autogrid_grid'] );
				break;
			case 'autogridColStart':
			case 'autogridColStop':
				$arrReturn = $this->getTemplateGroup('ce_autogrid_col',array(),'ce_autogrid_col');
				unset( $arrReturn['ce_autogrid_col'] );
				break;
			default: 
				return $this->getTemplateGroup('ce_');
				break;
		}

		return $arrReturn;
	}
	
	
	/**
	 * Adjust the backend limit to avoid open grid previews
	 */
	public function adjustLimit()
	{
		$intLimit = $GLOBALS['TL_CONFIG']['resultsPerPage'];
		
		$strPtable = 'tl_article';
		if( \Contao\Input::get('do') == 'news' )
		{
			$strPtable = 'tl_news';
		}
		else if( \Contao\Input::get('do') == 'calendar' )
		{
			$strPtable = 'tl_calendar_events';
		}

		// fetch all records to be displayed
		$objModels = \Contao\ContentModel::findBy( array('pid=?','ptable=?'), array(\Contao\Input::get('id'),$strPtable), array('limit'=>$intLimit) );
		if($objModels === null)
		{
			return;
		}
		
		$arrIds = array();
		foreach($objModels as $i => $objModel)
		{
			$arrIds[] = $objModel->id;
			
			if( in_array($objModel->type, array('autogridGridStart','autogridRowStart')) === true )
			{
				$objStop = ContentModel::findSiblingByTypeAndPidAndPtableAndAutoGridId($objModel->type, $objModel->pid, $objModel->ptable, $objModel->autogrid_id);
				
				$objBetween = ContentModel::findBetween($objModel->id, $objStop->id);
				if($objBetween)
				{
					$arrIds = array_merge($arrIds,$objBetween->fetchEach('id'));
				}
				$arrIds[] = $objStop->id;
			}
		}
		
		$intCount = count( array_unique($arrIds) );
		
		if( $intLimit < $intCount )
		{
			$GLOBALS['TL_CONFIG']['resultsPerPage'] = $intCount;
			
			// log
			if($GLOBALS['PCT_AUTOGRID']['debug'] === true)
			{
				\Contao\System::log('Auto adjust backend list limit to fit grid view: tl_content.pid='.\Contao\Input::get('id'),'AutoGrid',TL_GENERAL);
			}
		}
	}
	
	
	/**
	 * List elements
	 * @param array
	 * @return string
	 *
	 * called from child_record_callback
	 */
	public function listRecord($arrRow)
	{
		$arrHelperCallback = $GLOBALS['PCT_AUTOGRID']['child_record_callback']['tl_content'];
		
		$objHelper = new $arrHelperCallback[0];
		$strBuffer = $objHelper->{$arrHelperCallback[1]}($arrRow);
		
		// @var object
		$objModel = new ContentModel();
		$objModel->setRow($arrRow);
		
		// deactivate autogrid for any element not being a wrapper element
		if( !in_array($objModel->type, $GLOBALS['PCT_AUTOGRID']['wrapperElements']) && (boolean)$objModel->autogrid === true)
		{
			// column started but not in a row || not in flex row in general
			if( ($GLOBALS['PCT_AUTOGRID']['autogridColStarted'] && !$GLOBALS['PCT_AUTOGRID']['autogridRowStarted']) || !$GLOBALS['PCT_AUTOGRID']['autogridRowStarted'])
			{
				$objModel->autogrid = 0;
				$this->setAutoGrid($objModel->id, false);
			}
		}
		
		// activate autogrid for wrapper elements
		if( in_array($objModel->type, $GLOBALS['PCT_AUTOGRID']['wrapperElements']) )
		{
			$objModel->autogrid = 1;
		}

		// do not manipulate elements outside of AG
		#if( (boolean)$objModel->autogrid === false )
		#{
		#	return $strBuffer;
		#}
		
		//--- Rows
		$start = 'autogridRowStart';
		$stop = 'autogridRowStop';
		
		if($objModel->type == $start)
		{
			$GLOBALS['PCT_AUTOGRID']['autogridRowStarted'] = true;

			$GLOBALS['PCT_AUTOGRID']['autogridRowsStart'][] = $objModel->id;
			
			// fetch the whole AG block
			// @var object
			$objModels = ContentModel::findAllByStartId($objModel->id);
			if($objModels !== null)
			{
				$arrIds = $objModels->fetchEach('id');
				
				$GLOBALS['PCT_AUTOGRID']['autogridGridPreviews'][$objModel->id] = array
				(
					'ids'	=> $arrIds,
					'start'	=> $objModel->id,
					'stop'	=> end( $arrIds ),
				);
			
				// generate backend grid view
				$objGridPreview = new \Contao\BackendTemplate('be_mod_grid_preview');
				$objGridPreview->setData( $objModel->row() );
				$objGridPreview->models = $objModels;
				$objGridPreview->flex = true;
				$strPreview = $objGridPreview->parse();
				
				$strPreview = \Contao\StringUtil::substrHtml($strPreview,strlen($strPreview));
			
				$GLOBALS['PCT_AUTOGRID']['autogridGridPreviews'][$objModel->id]['html'] = $strPreview;
			}
		}
		else if($objModel->type == $stop)
		{
			$GLOBALS['PCT_AUTOGRID']['autogridRowsStop'][] = $objModel->id;

			unset($GLOBALS['PCT_AUTOGRID']['autogridRowStarted']);
		}
		//--
		
		//-- Cols
		$start = 'autogridColStart';
		$stop = 'autogridColStop';
		if($objModel->type == $start)
		{
			$GLOBALS['PCT_AUTOGRID']['autogridColStarted'] = true;

			// start a new collection
			$GLOBALS['PCT_AUTOGRID']['autogridColsStart'][] = $objModel->id;
		}
		else if($objModel->type == $stop)
		{
			$GLOBALS['PCT_AUTOGRID']['autogridColsStop'][] = $objModel->id;

			unset($GLOBALS['PCT_AUTOGRID']['autogridColStarted']);
		}
		
		//-- Grids
		$start = 'autogridGridStart';
		$stop = 'autogridGridStop';
		
		if($objModel->type == $start)
		{
			$GLOBALS['PCT_AUTOGRID']['autogridGridStarted'] = true;

			// start a new collection
			$GLOBALS['PCT_AUTOGRID']['autogridGridStart'][] = $objModel->id;
			
			// fetch the whole AG block
			// @var object
			$objModels = ContentModel::findAllByStartId($objModel->id);

			if($objModels !== null)
			{
				$arrIds = $objModels->fetchEach('id');
				
				$GLOBALS['PCT_AUTOGRID']['autogridGridPreviews'][$objModel->id] = array
				(
					'ids'	=> $arrIds,
					'start'	=> $objModel->id,
					'stop'	=> end( $arrIds ),
				);
			
				// generate backend grid view
				$objGridPreview = new \Contao\BackendTemplate('be_mod_grid_preview');
				$objGridPreview->setData( $objModel->row() );
				$objGridPreview->models = $objModels;
				$objGridPreview->grid = true;
				$strPreview = $objGridPreview->parse();
				
				$strPreview = \Contao\StringUtil::substrHtml($strPreview,strlen($strPreview));
			
				$GLOBALS['PCT_AUTOGRID']['autogridGridPreviews'][$objModel->id]['html'] = $strPreview;
			}
		}
		else if($objModel->type == $stop)
		{
			$GLOBALS['PCT_AUTOGRID']['autogridGridStop'][] = $objModel->id;

			unset($GLOBALS['PCT_AUTOGRID']['autogridGridStarted']);
		}

		// @var object
		$objJsTemplate = new \Contao\BackendTemplate('be_js_autogrid');
		$objJsTemplate->setData( $objModel->row() );
		$objJsTemplate->context = 'listElement';
		if( $GLOBALS['PCT_AUTOGRID']['autogridRowStarted'] && $GLOBALS['PCT_AUTOGRID']['autogridColStarted'] && $objModel->type == 'autogridColStart' )
		{
			$objJsTemplate->blnShowButtons = true;
		}
		
		// add autogrid to template
		AutoGrid::addToTemplate($objJsTemplate);
		
		// collect a couple input related classes
		$arrInputClasses = array( \Contao\Input::get('act'), \Contao\Input::get('mode') );
		$objJsTemplate->input_classes = implode(' ', array_unique( array_filter( $arrInputClasses ) ) );

		// append the JS helper stuff
		$strBuffer .= $objJsTemplate->parse();
		
		if($objModel->autogrid_id && $GLOBALS['PCT_AUTOGRID']['debug'] === true)
		{
			// show reference
			$strBuffer .= '<div>'.$objModel->id.' <-> '.$objModel->autogrid_id .'</div>';
		}

		//---
		// !toggle autogrid in flex rows
		$start = 'autogridRowStart';
		$stop = 'autogridRowStop';
		
		if($objModel->type == $start)
		{
			$blnReload = false;
			// fetch the whole AG block
			// @var object
			$objModels = ContentModel::findAllByStartId($objModel->id);
			if($objModels !== null)
			{
				foreach($objModels as $model)
				{
					if( in_array($model->type, $GLOBALS['PCT_AUTOGRID']['wrapperElements']) || (boolean)$model->autogrid === true )
					{
						continue;
					}

					$this->setAutoGrid($model->id, true);

					// flag reload
					$blnReload = true;
				}		
			}
			
			if($blnReload === true)
			{
				\Contao\Controller::reload();
			}
		}

		return $strBuffer;	
	}
	
	
	/**
	 * Show the name of the grid preset as backend information to the user
	 * @param object
	 */
	public function showInfoInGridPresets($objDC)
	{
		if( $objDC->activeRecord === null )
		{
			$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		}
		
		if( in_array(\Contao\Input::get('act'),array('edit')) && $objDC->activeRecord->type == 'autogridGridStart' && !empty($objDC->activeRecord->autogrid_grid) )
		{
			$arrPreset = \PCT\AutoGrid\GridPreset::getGridPreset($objDC->activeRecord->autogrid_grid);
			$name = $GLOBALS['TL_LANG']['autogrid_grid'][ $objDC->activeRecord->autogrid_grid ] ?: $objDC->activeRecord->autogrid_grid;
			if($arrPreset['label'])
			{
				$name = $arrPreset['label'];
			}
			\Contao\Message::addInfo( sprintf($GLOBALS['TL_LANG']['AUTOGRID_MESSAGES']['gridInfo'],$name) );

			// show additional info when user settings varies from preset
			if( $objDC->activeRecord->autogrid_css != $arrPreset['grid']['desktop'] )
			{
				\Contao\Message::addInfo( sprintf($GLOBALS['TL_LANG']['AUTOGRID_MESSAGES']['gridInfo_customSettings'], $objDC->activeRecord->autogrid_css, $arrPreset['grid']['desktop']) );
			}
		}
	}
	
		
	/**
	 * Delete the sibling STOP element and AG Elements between when a START element is being deleted
	 * @param object
	 * @param integer
	 * 
	 * @called from ondelete_callback
	 */
	public function deleteSiblings($objDC,$intUndo)
	{
		// do not trigger in multi modes
		if( in_array(\Contao\Input::get('act'), array('select','deleteAll') ) )
		{
			return;
		}

		if( $objDC->activeRecord === null )
		{
			$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		}
		
		$strType = $objDC->activeRecord->type;
		if( array_key_exists($strType, $GLOBALS['PCT_AUTOGRID']['deleteSiblingOnDelete']) === false )
		{
			return;
		}
				
		$objStop = ContentModel::findSiblingByTypeAndPidAndPtableAndAutoGridId($strType,$objDC->activeRecord->pid,$objDC->activeRecord->ptable,$objDC->activeRecord->autogrid_id);
		if( $objStop === null )
		{
			return;
		}
		
		$arrEffected = $GLOBALS['PCT_AUTOGRID']['deleteNestedOnDelete'][$strType] ?: array();

		$objBetween = ContentModel::findBetween($objDC->id, $objStop->id);
		if($objBetween !== null)
		{
			while($objBetween->next())
			{
				// do not delete elements depending on start element
				if( in_array($objBetween->type, $arrEffected) === false )
				{
					// turn AG off
					$model = ContentModel::findByPk($objBetween->id);
					$model->__set('autogrid',0);
					$model->save();
					continue;
				}

				// check if record is part of AG and delete it
				if( in_array($objBetween->type, $GLOBALS['PCT_AUTOGRID']['wrapperElements']) )
				{
					$dc = clone($objDC);
					$dc->activeRecord = null;
					$dc->intId = $objBetween->id;
					$dc->delete(true);
				}
			}
		}
		
		// delete STOP
		$dc = clone($objDC);
		$dc->activeRecord = null;
		$dc->intId = $objStop->id;
		$dc->delete(true);
	}
	
	
	/**
	 * Update the autogrid references after copy events
	 * @param integer
	 * @param object
	 * 
	 * @called from oncopy_callback
	 */
	public function storeNewElements($intId, $objDC)
	{
		$objDC->activeRecord = \Contao\ContentModel::findByPk($intId);
		
		$strType = $objDC->activeRecord->type;
		
		if( in_array($strType, $GLOBALS['PCT_AUTOGRID']['wrapperElements']) === false )
		{
			return;
		}
		
		$arrSession = $this->Session->get('NEW_AG_ELEMENTS');
		if( !is_array($arrSession) )
		{
			$arrSession = array();
		}
		
		$arrSession[$objDC->table][$strType][] = $intId;
		
		$this->Session->set('NEW_AG_ELEMENTS',$arrSession);
	}
	
	
	/**
	 * Update the newly created AG elements after copy
	 * @param object
	 */
	public function updateNewElements($objDC)
	{
		$arrSession = $this->Session->get('NEW_AG_ELEMENTS');
		
		if( empty($arrSession[$objDC->table]) === true )
		{
			return;
		}
		
		if( is_array($arrSession[$objDC->table]) === false )
		{
			return;
		}
		
		$arrElements = $arrSession[$objDC->table];
		
		// update the references in the grid
		$arrUpdated = \PCT\AutoGrid\Controller::updateElementsWithIdentifyier($objDC->table,$arrElements);
		
		// log
		if( empty($arrUpdated) === false )
		{
			\Contao\System::log('Updated AutoGrid elements (oncopy) in '.$objDC->table.': '.implode(',', $arrUpdated),__METHOD__,TL_GENERAL);
		}
		unset($arrUpdated);
		
		//clear the session
		$arrSession[$objDC->table] = array();
		$this->Session->set('NEW_AG_ELEMENTS',$arrSession);
	}
	
	
	/**
	 * Observe the clipboard and add sibling AG elements to the clipboard
	 * @param object
	 * 
	 * @called from onload_callback
	 */
	public function observeClipboard($objDC)
	{
		if( $objDC->activeRecord === null )
		{
			$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		}
	
		$strType = $objDC->activeRecord->type;
	
		if( !in_array($strType, array('autogridGridStart','autogridColStart','autogridRowStart')) )		
		{
			return;
		}
		
		$arrClipboard = $this->Session->get('CLIPBOARD');
		#$strMode = $arrClipboard[$objDC->table]['mode'];
		
		$arrIds = $arrClipboard[$objDC->table]['id'];
		
		if( isset($arrClipboard[$objDC->table]['id']) && !is_array($arrClipboard[$objDC->table]['id']) )
		{
			$arrIds = array( $arrClipboard[$objDC->table]['id'] );
		}
				
		if( empty($arrIds) === true )
		{
			return;
		}
				
		foreach($arrIds as $id)
		{
			$dc = clone($objDC);
			$dc->activeRecord = \Contao\ContentModel::findByPk($id);
			
			$objStop = ContentModel::findSiblingByTypeAndPidAndPtableAndAutoGridId($strType,$dc->activeRecord->pid, $dc->activeRecord->ptable, $dc->activeRecord->autogrid_id);
			if($objStop === null)
			{
				continue;
			}
			
			$objBetween = ContentModel::findBetween($dc->id,$objStop->id);
			if($objBetween !== null)
			{
				$arrIds =  array_merge($arrIds, $objBetween->fetchEach('id') );
			}
			
			// add stop
			$arrIds[] = $objStop->id;
		}
		
		// cut
		if( \Contao\Input::get('mode') == 'cut' || \Contao\Input::get('act') == 'cut' )
		{
			$arrClipboard[$objDC->table]['id'] = $arrIds;
			$arrClipboard[$objDC->table]['mode'] = 'cutAll';
			$this->Session->set('CLIPBOARD',$arrClipboard);
			// simulate multiple cut
			if( \Contao\Input::get('act') == 'cut' )
			{
				$url = \Contao\StringUtil::decodeEntities( $this->addToUrl('act=cutAll') );
				$this->redirect( $url );
			}
		}
		// copy
		if( \Contao\Input::get('mode') == 'copy' || \Contao\Input::get('act') == 'copy' )
		{
			$arrClipboard[$objDC->table]['id'] = $arrIds;
			$arrClipboard[$objDC->table]['mode'] = 'copyAll';
			$this->Session->set('CLIPBOARD',$arrClipboard);
			// simulate multiple copy
			if( \Contao\Input::get('act') == 'copy' )
			{
				$url = \Contao\StringUtil::decodeEntities( $this->addToUrl('act=copyAll') );
				$this->redirect( $url );
			}
		}
	}
	
	
	/**
	 * Create sibling row-stop element
	 * @param integer
	 * @param object
	 */
	public function createSiblingStopElement($objDC)
	{
		$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		
		// no start element or just a submitOnChange
		if( array_key_exists($objDC->activeRecord->type, $GLOBALS['PCT_AUTOGRID']['createSiblingOnCreate']) === false || \Contao\Input::post('SUBMIT_TYPE') == 'auto' )
		{
			return;
		}
		
		// element is flagged, check if it has a stop sibling already
		$objStop = null;
		if( $objDC->activeRecord->autogrid_id > 0 )
		{
			$objStop = ContentModel::findSiblingByTypeAndPidAndPtableAndAutoGridId($objDC->activeRecord->type,$objDC->activeRecord->pid,$objDC->activeRecord->ptable,$objDC->activeRecord->autogrid_id);		
		}
		
		if( $objStop !== null )
		{
			return;
		}
		
		$strStop = $GLOBALS['PCT_AUTOGRID']['createSiblingOnCreate'][$objDC->activeRecord->type];
		
		$arrSet = $objDC->activeRecord->row();
		unset($arrSet['id']);
		// reference the start element
		$arrSet['type'] = $strStop;
		$arrSet['sorting'] = $objDC->activeRecord->sorting + 1;
		$arrSet['autogrid_id'] = $objDC->id;
		$this->Database->prepare("INSERT INTO ".$objDC->table." %s")->set($arrSet)->execute();
		
		// update this element
		$objDC->activeRecord->__set('autogrid_id',$objDC->id);
		$objDC->activeRecord->save();
	}


	/**
	 * oncopy event listener
	 * @param integer
	 * @param object
	 * 
	 * called from oncopy_callback
	 */
	public function oncopy_toggleAutoGrid($intId,$objDC)
	{
		// do not effect the block-copy-event from AG
		$arrSession = $this->Session->get('NEW_AG_ELEMENTS');
		if( is_array($arrSession[$objDC->table]) === true && empty($arrSession[$objDC->table]) === false && \Contao\Input::get('act') == 'copyAll' )
		{
			return;
		}
		$dc = clone($objDC);
		$dc->id = $intId;
		$this->toggleAutoGrid($dc);
	}

	
	/**
	 * oncut event listener
	 * @param integer
	 * @param object
	 * 
	 * called from oncut_callback
	 */
	public function oncut_toggleAutoGrid($objDC)
	{
		$this->toggleAutoGrid( clone($objDC) );
	}


	/**
	 * Enable or disable AG on contao events or data container information
	 * @param object
	 */
	public function toggleAutoGrid($objDC=null)
	{
		if( $objDC === null || (int)$objDC->id < 1 )
		{
			return;
		}
		
		$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);

		$strSession = 'TOGGLE_AG_ELEMENTS_'.$objDC->table;
		// get further ids that should be ignored
		$arrIgnore = $this->Session->get($strSession ) ?: array();
		if( is_array($arrIgnore) === false )
		{
			$arrIgnore = array();
		}
		// do not manipulate elements effected when an AG Start elements is being called
		if( in_array($objDC->activeRecord->type,$GLOBALS['PCT_AUTOGRID']['startElements']) === true )
		{
			$objEffected = ContentModel::findAllByStartId($objDC->id);
			// store effected records in session
			if( $objEffected !== null )
			{
				$arrIgnore = array_unique( array_merge($arrIgnore, $objEffected->fetchEach('id')) );
				$this->Session->set($strSession,$arrIgnore);
			}
		}
		// if the element should no be effected return here
		if( in_array($objDC->id, $arrIgnore) === true )
		{
			// reduce array
			unset($arrIgnore[ array_search($objDC->id, $arrIgnore) ]);
			// update session
			$this->Session->set($strSession,$arrIgnore);
			return;
		}

		// check if element is wrapped between autogridRows or autogridCols
		$arrOptions = array('column'=> array($this->Database->findInSet('type',array('autogridRowStart,autogridColStart')) ) );

		$objStart = ContentModel::findPreviousById($objDC->id, $arrOptions);
		#$objStart = $this->Database->prepare("SELECT * FROM ".$objDC->table." WHERE pid=? AND ptable=? AND sorting <= ? AND ".$this->Database->findInSet('type',array('autogridRowStart,autogridColStart'))." ORDER BY sorting ASC")->limit(1)->execute($objDC->activeRecord->pid,$objDC->activeRecord->ptable,$objDC->activeRecord->sorting);
		
		// no start element before and AG is active -> disable
		if( (int)$objStart->id < 1 && (boolean)$objDC->activeRecord->autogrid === true )
		{
			return $this->setAutoGrid($objDC->id, false);
		}
		
		// find stop element and elements between
		$objStop = ContentModel::findSiblingByTypeAndPidAndPtableAndAutoGridId($objStart->type,$objStart->pid,$objStart->ptable,$objStart->autogrid_id);		
		$objBetween = ContentModel::findBetween($objStart->id,$objStop->id,array(),true);

		$arrBetween = array();
		if( $objBetween !== null )
		{
			$arrBetween = $objBetween->fetchEach('id');
		}

		// elements is between grid columns
		if( in_array($objDC->id, $arrBetween) === true && $objStart->type == 'autogridColStart'  )
		{
			return $this->setAutoGrid($objDC->id, false);
		}
		// elements is between flex rows
		else if( in_array($objDC->id, $arrBetween) === true && $objStart->type == 'autogridRowStart'  )
		{
			return $this->setAutoGrid($objDC->id, true);
		}
		// between nothing
		else
		{
			return $this->setAutoGrid($objDC->id, false);
		}
	}


	/**
	 * Update the AutoGrid state for a certain record
	 * @param integer
	 * @param boolean
	 * @param array
	 */
	public function setAutoGrid($intId, $blnValue, $arrSet=array())
	{
		// @var object
		$objModel = \Contao\ContentModel::findByPk($intId);
		$objModel->__set('autogrid',$blnValue);
		if( count($arrSet) > 0 )
		{
			foreach($arrSet as $k => $v)
			{
				$objModel->__set($k,$v);
			}
		}
		$objModel->save();
		return $blnValue;
	}


	/**
	 * Active AutoGrid for new elements inbetween flex rows or deactivate inbetween cols
	 * @param integer
	 * @param object
	 * 
	 * called from onsubmit_callback
	 */
	public function activateAutoGridInFlexRows($objDC)
	{
		$objModel = ContentModel::findByPk($objDC->id);
		
		// fetch closest start element before
		$arrOptions = array('column'=> array($this->Database->findInSet('type',array('autogridRowStart,autogridColStart')) ) );
		$objStart = ContentModel::findPreviousById($objDC->id, $arrOptions);
		
		// deactivate autogrid inbetween autogridCols
		if( $objStart->type == 'autogridColStart' )
		{
			$objBetween = ContentModel::findAllByStartId($objStart->id);
			if( $objBetween !== null )
			{
				if( in_array($objDC->activeRecord->id, $objBetween->fetchEach('id') ) && (boolean)$objModel->autogrid === true )
				{
					// show backend info
					\Contao\Message::addInfo($GLOBALS['TL_LANG']['AUTOGRID_MESSAGES']['colInfo']);

					// @var object
					$objModel->__set('autogrid',0);
					$objModel->save();
				}
			}
		}
		// activate autogrid inbetween autogridCols
		else if($objStart->type == 'autogridRowStart' )
		{
			$objBetween = ContentModel::findAllByStartId($objStart->id);
			if( $objBetween !== null )
			{
				if( in_array($objDC->activeRecord->id, $objBetween->fetchEach('id') ) && (boolean)$objModel->autogrid === false )
				{
					// show backend info
					\Contao\Message::addInfo($GLOBALS['TL_LANG']['AUTOGRID_MESSAGES']['flexInfo']);

					// @var object
					$objModel->__set('autogrid',1);
					if( empty($objModel->autogrid_css) )
					{
						$objModel->__set('autogrid_css','col_12');
					}
					$objModel->save();
				}
			}
		}
	}


	/**
	 * Toggle autogrid block visibility listener
	 * @param object
	 */
	public function toggleBlockAjaxListener($objDC)
	{
		if( (boolean)\Contao\Environment::get('isAjaxRequest') === false || \Contao\Input::get('action') != 'toggleAutoGridBlockVisibility' || (int)\Contao\Input::get('elem') < 1 )
		{
			return;
		}

		$objElements = ContentModel::findAllByStartId( (int)\Contao\Input::get('elem') );
		if( $objElements === null )
		{
			return;
		}
		
		$blnVisible =  ( (int)\Contao\Input::get('state') == 0 ? false : true );
		$arrElements = $objElements->fetchEach('id');
		$intTime = time();

		// call the tl_content.toggleVisibility for each element
		foreach($arrElements as $id)
		{
			$dc = clone($objDC);
			$dc->intId = $dc->id = $id;
			
			// Check the field access
			if (!$this->User->hasAccess('tl_content::invisible', 'alexf'))
			{
				\Contao\System::log('Not enough permissions to publish/unpublish content element ID "'. $id.'"', __METHOD__, TL_ERROR);
				
				if( version_compare(VERSION,'4.4','>=') )
				{
					throw new \Contao\CoreBundle\Exception\AccessDeniedException('Not enough permissions to show/hide content element ID ' . $id . '.');
				}
				
				\Contao\Controller::redirect('contao/main.php?act=error');
			}

			// @var object Version
			$objVersions = new \Contao\Versions('tl_content', $id);
			$objVersions->initialize();

			// update the record
			// @var object
			$objModel = ContentModel::findByPk( $id );
			$objModel->__set('invisible',($blnVisible === true ? '1' : '') );
			$objModel->__set('tstamp',$intTime);
			$objModel->save();

			// Trigger the save_callback
			if (is_array($GLOBALS['TL_DCA']['tl_content']['fields']['invisible']['save_callback']))
			{
				foreach ($GLOBALS['TL_DCA']['tl_content']['fields']['invisible']['save_callback'] as $callback)
				{
					$blnVisible = \Contao\System::importStatic($callback[0])->{$callback[1]}($blnVisible,$dc);
				}
			}

			// create new version
			$objVersions->create();
		};
	}
}