<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @subpackage	AttributeInclude
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Attributes;

/**
 * Imports
 */
use PCT\CustomElements\Core\Attribute as Attribute;
use PCT\CustomElements\Helper\ControllerHelper as ControllerHelper;

/**
 * Class file
 * IncludeElement
 */
class IncludeElement extends Attribute
{
	/**
	 * Return the field definition
	 * @return array
	 */
	public function getFieldDefinition()
	{
		$arrReturn = array
		(
			'label'			=> array( $this->get('title'),$this->get('description') ),
			'exclude'		=> true,
			'inputType'		=> 'text',
			'eval'			=> $this->getEval(),
			'sql'			=> "int(10) NOT NULL default '0'",
		);
		
		$arrReturn['eval']['chosen'] = true;
		
		$blnInstallTool = false;
		if(strlen(strpos(\Contao\Environment::getInstance()->scriptName, '/contao/install.php')) > 0 || strlen(strpos(\Contao\Environment::getInstance()->requestUri, '/contao/install')) > 0)
		{
			$blnInstallTool = true;
		}
		
		// fetch include options
		if(strlen($this->get('include_type')) > 0 && TL_MODE == 'BE' && !$blnInstallTool)
		{
			$arrReturn['inputType'] = 'select';
			#$arrReturn['eval']['chosen'] = true;
			
			if(!is_array($GLOBALS['TL_DCA']['tl_content']['config']))
			{
				ControllerHelper::callstatic('loadDataContainer',array('tl_content'));
			}
			
			$tableContent = new \tl_content();
			$dc = new \PCT\CustomElements\Helper\DataContainerHelper();
			$dc->table = 'tl_content';
			$dc->id = \Contao\Input::get('id');
			
			switch($this->get('include_type'))
			{
				case 'article': 
					$options = $tableContent->getArticles($dc);
					break;
				case 'content': 
					$options = $tableContent->getAlias($dc);
					break;
				case 'form': 
					$options = $tableContent->getForms($dc);
					break;
				case 'module':
					$options = $tableContent->getModules($dc);
					break;
				default:
					break;
			}
			$arrReturn['options'] = $options;
		}
		
		return $arrReturn;
	}
	
	
	/**
	 * Parse widget callback, render the attribute in the backend
	 * @param object
	 * @return string
	 */
	public function parseWidgetCallback($objWidget)
	{
	   if(!$this->get('include_item'))
	   {
	   		return $objWidget->parse();
	   }
	   
	   return '';
	}

	/**
	 * Render the attribute and return html
	 * @return array
	 */
	public function renderCallback($strField,$varValue,$objTemplate,$objAttribute)
	{
		$item = $this->get('include_item');
		
		if(!$item)
		{
			$item = $varValue;
		}
		
		$strBuffer = '';
		switch($this->get('include_type'))
		{
			case 'article':
				$strBuffer = ControllerHelper::callstatic('replaceInsertTags',array('{{insert_article::'.$item.'}}'));
				break;
			case 'content': 
				$strBuffer = ControllerHelper::callstatic('replaceInsertTags',array('{{insert_content::'.$item.'}}'));
				break;
			case 'form': 
				$strBuffer = ControllerHelper::callstatic('replaceInsertTags',array('{{insert_form::'.$item.'}}'));
				break;
			case 'module':
				$strBuffer = ControllerHelper::callstatic('replaceInsertTags',array('{{insert_module::'.$item.'}}'));
				break;
			default:
				break;
		}
		
		$objTemplate->value = $strBuffer;
		
		return $objTemplate->parse();
	}
}