<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @subpackage	Attribute IncludeElement
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Attributes\IncludeElement;

/**
 * Class file
 * TableCustomElementAttribute
 */
class TableCustomElementAttribute extends \Contao\Backend
{
	/**
	 * Modify palette
	 * @param object
	 */
	public function modifyPalette($objDC)
	{
		$objActiveRecord = $objDC->activeRecord;
		$strModel = \Contao\Model::getClassFromTable($objDC->table);
		if($objActiveRecord === null && class_exists($strModel))
		{
			$objActiveRecord = $strModel::findByPk($objDC->id);
		}
		else if($objActiveRecord === null && !class_exists($strModel))
		{
			$objActiveRecord = \Contao\Database::getInstance()->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		}
				
		switch($objActiveRecord->include_type)
		{
			case 'article': 
				$GLOBALS['TL_DCA'][$objDC->table]['fields']['include_item']['wizard'][] = array('tl_content', 'editArticle');
				break;
			case 'content': 
				$GLOBALS['TL_DCA'][$objDC->table]['fields']['include_item']['wizard'][] = array('tl_content', 'editAlias');
				break;
			case 'form': 
				$GLOBALS['TL_DCA'][$objDC->table]['fields']['include_item']['wizard'][] = array('tl_content', 'editForm');
				break;
			case 'module':
				$GLOBALS['TL_DCA'][$objDC->table]['fields']['include_item']['wizard'][] = array('tl_content', 'editModule');
				break;
			default:
				break;
		}
	}

	
	/**
	 * Return possible include elements as array
	 * @param object
	 * @return array
	 */
	public function getIncludeItem($objDC)
	{
		$objSession = \Contao\Session::getInstance();
		
		$objActiveRecord = $objDC->activeRecord;
		$strModel = \Contao\Model::getClassFromTable($objDC->table);
		if($objActiveRecord === null && class_exists($strModel))
		{
			$objActiveRecord = $strModel::findByPk($objDC->id);
		}
		else
		{
			$objActiveRecord = \Contao\Database::getInstance()->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		}

		$arrReturn = array();
		if(strlen($objSession->get('pct_customelement_attribute_include_type')) && $objSession->get('pct_customelement_attribute_include_type') != $objActiveRecord->include_type)
		{
			\Contao\Database::getInstance()->prepare("UPDATE ".$objDC->table." %s WHERE id=?")->set(array('include_item'=>0))->execute($objDC->id);
			$objSession->set('pct_customelement_attribute_include_type',$objActiveRecord->include_type);
			$this->reload();
		}
		
		// load tl_content class
		if(!is_array($GLOBALS['TL_DCA']['tl_content']['config']))
		{
			\PCT\CustomElements\Helper\ControllerHelper::callstatic('loadDataContainer',array('tl_content'));
		}
		
		$tableContent = new \tl_content();
		switch($objActiveRecord->include_type)
		{
			case 'article': 
				$arrReturn = $tableContent->getArticles($objDC);
				break;
			case 'content': 
				$arrReturn = $tableContent->getAlias($objDC);
				break;
			case 'form': 
				$arrReturn = $tableContent->getForms($objDC);
				break;
			case 'module':
				$arrReturn = $tableContent->getModules($objDC);
				break;
			default:
				break;
		}
		
		// store include type
		$objSession->set('pct_customelement_attribute_include_type',$objActiveRecord->include_type);
		
		return $arrReturn;
	}
}