<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2014
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  pct_customelements
 * @link  http://contao.org
 * @license  http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Backend;

/**
 * Class file
 * Import
 */
class Import extends \Contao\Backend
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'be_page_import';

	/**
	 * Default preview image setting
	 * @var array
	 */
	protected $arrPreviewSize = array(0,80,'proportional');

	/**
	 * Import data array
	 * @var array
	 */
	protected $arrImportData = array();

	/**
	 * Import data array
	 * @var array
	 */
	protected $arrImportChain = array();
	
	/**
	 * Session name
	 * @var string
	 */
	protected $strSession = 'tl_pct_customelement_import';
	
	/**
	 * Flag to also include the old sql based templates
	 * @var boolean
	 */
	protected $blnIncludeSqlTemplates = false;
	
	

	/**
	 * Initialize and authenticate backend user
	 */
	public function __construct()
	{
		$objUser = \Contao\BackendUser::getInstance();
		if($objUser->id < 1)
		{
			$objUser->authenticate();
		}
		parent::__construct();

		// load language files
		$this->loadLanguageFile('default');
		$this->loadLanguageFile('import');

		// include styles
		$GLOBALS['TL_CSS'][] = PCT_CUSTOMELEMENTS_PATH.'/assets/css/styles.css';
	}

	/**
	 * Create the backend inferface
	 * @return string
	 */
	public function createInterface()
	{
		$objSession = \Contao\Session::getInstance();
		if(version_compare(VERSION, '4','>='))
		{
			$objSession = \Contao\System::getContainer()->get('session');
		}
		$objTemplate = new \Contao\BackendTemplate($this->strTemplate);

		// back button
		$objTemplate->back = '<a class="header_back" onclick="Backend.getScrollOffset()" accesskey="b" title="" href="'.$this->getReferer().'">'.$GLOBALS['TL_LANG']['MSC']['goBack'].'</a>';
		
		// cehck if old .sql templates are installed
		if(is_dir(TL_ROOT.'/'.$GLOBALS['PCT_CUSTOMELEMENTS']['path_to_import']))
		{
			$arrOldSqlTemplates = scan(TL_ROOT.'/'.$GLOBALS['PCT_CUSTOMELEMENTS']['path_to_import']);
			if(is_array($arrOldSqlTemplates) && !empty($arrOldSqlTemplates))
			{
				if(in_array('tl_pct_customelement.sql', $arrOldSqlTemplates))
				{
					$this->blnIncludeSqlTemplates = true;
				}
			}
		}
		
		$arrOptions = $this->getImportOptionsFromTemplate();
		
		if($this->blnIncludeSqlTemplates)
		{
			$arrOptions = array_merge($arrOptions, $this->getImportOptions() ?: array() );	
		}
		
		// template selection widget
		$arrData = array
		(
			'label'                   => &$GLOBALS['TL_LANG']['page_import']['import_select'],
			'exclude'                 => true,
			'inputType'               => 'checkbox',
			'options'      				=> $arrOptions,
			'reference'      			=> $GLOBALS['TL_LANG']['page_import']['import_select'],
			'eval'                    => array('tl_class'=>'w50','multiple'=>true,'includeBlankOption'=>0),
		);
		
		$strClass = $GLOBALS['BE_FFL'][$arrData['inputType']];
		if(!class_exists($strClass))
		{
			return '';
		}
		
		$objWidget = new $strClass($this->prepareForWidget($arrData, 'import_select', 'hallo', 'import_select'));

		$objTemplate->selectionMenu = (empty($arrOptions) ? $GLOBALS['TL_LANG']['page_import']['done'] : $objWidget->parse());

		
		$arrSession = $objSession->get($this->strSession);
		if(!$arrSession)
		{
			$arrSession = array();
		}
		
		$objTemplate->imported = $arrSession['imported'];
		$objTemplate->hasError = false;
		
		if(array_key_exists('error', $arrSession))
		{
			$objTemplate->hasError = true;
			$arrError = $arrSession['error'];
			$strError = implode('</br>', $arrError);
			$objTemplate->error = $strError;	
		}
		
		// perform import and reload page to prevend reposts
		if(\Contao\Input::post('run_import'))
		{
			$arrPost = \Contao\Input::post('import_select');
			if(empty($arrPost))
			{
				$this->reload();
			}
			
			foreach($arrPost as $f)
			{
				$arrTemplates = $this->getImportOptionsFromTemplate();
				if(array_key_exists($f, $arrTemplates) )
				{
					$this->importTemplate($f);
				}
				else
				{
					$this->runImport();
				}
			}
			
			
			$this->reload();
		}

		// submit form
		$objTemplate->action = \Contao\Environment::get('request');
		$objTemplate->import_value = $GLOBALS['TL_LANG']['page_import']['import_value'];

		// remove the session when displayed once
		$objSession->remove($this->strSession);

		return $objTemplate->parse();
	}
	
	protected function setImportChain()
	{
		// import chain
		$this->arrImportChain = array
		(
			'tl_pct_customelement' => array
			(
				'master' => 'alias',
				'filter' => array
				(
					array
					(
						'field'  => 'alias',
						'operation' => 'IN',
						'values' => \Contao\Input::post('import_select'),
					),
				),
			),
			'tl_pct_customelement_group' => array
			(
			   'reference' 		=> array('pid','tl_pct_customelement.id'),
			   'master' 		=> 'id',
			),
			'tl_pct_customelement_attribute' => array
			(
			   'reference'  	=> array('pid','tl_pct_customelement_group.id'),
			   'master' 		=> 'id',
			),
		);
		
		// HOOK
		\PCT\CustomElements\Core\Hooks::callstatic('importChainHook',array($this));
	}
	
	
	/**
	 * Run import from template file
	 */
	protected function importTemplate($strTemplate)
	{
		$arrSession = \Contao\Session::getInstance()->get($this->strSession);
		
		$strFile = $GLOBALS['PCT_CUSTOMELEMENTS']['path_to_export'].'/'.$strTemplate.'.html5';
		$objFile = new \File($strFile,true);
		
		$alias = str_replace(array('import_','.html5'), array('',''), basename($strFile));
		\Contao\Session::getInstance()->set('current_import_alias',$alias);
		
		// remember the original template file content
		$strContent = $objFile->getContent();
		
		// inject a caller for the import method and pass the import array to it
		$strTempContent = $strContent;
		if(strlen(strpos($strContent, '$objImport->')) < 1)
		{
			$strTempContent .= '
			$objImport = new \PCT\CustomElements\Backend\Import(); $objImport->runImportFromTemplate($arrImport);';
			$objFile->putContent($strFile,$strTempContent);
		}
		
		$objTemplate = new \FrontendTemplate($strTemplate);
		$objTemplate->parse();
		
		// reset the file to its original content
		$objFile->putContent($strFile,$strContent);
	}
	
	
	/**
	 * Run import from template
	 * @param array
	 */
	public function runImportFromTemplate($arrImport)
	{
		if(empty($arrImport) || !is_array($arrImport))
		{
			return false;
		}
		
		$arrInserts = array_filter(explode('INSERT INTO', $arrImport['sql']),'strlen');
		if(count($arrInserts) < 1)
		{
			return;
		}
				
		$strAlias = \Contao\Session::getInstance()->get('current_import_alias');
		
		$objDatabase = \Contao\Database::getInstance();
	
		$arrImportData = array();
		foreach($arrInserts as $strInsert)
		{
			if(strlen($strInsert) < 1)
			{
				continue;
			}
			
			$strInsert = "INSERT INTO ".$strInsert;
			$strInsert = preg_replace('/^[#-]+/', '', trim($strInsert));
			preg_match('/^[^`]*`([^`]+)`/', $strInsert,$arrFirst);
			
			$strTable = $arrFirst[1];
			
			// check if table exists
			if(!$objDatabase->tableExists($strTable))
			{
				continue;
			}
			
			$strInsert = preg_replace('/'.$arrFirst[0].'/', '', $strInsert);
			
			$arrSecond = explode(' VALUES ', $strInsert);
			
			// sort out empty value parts
			$arrSecond[1] = str_replace('``', '` `', $arrSecond[1]);
			
			preg_match_all('/`([^`]+)`/', $arrSecond[0],$arrFields);
			preg_match_all('/`([^`]+)`/', $arrSecond[1],$arrValues);
			
			$arrFields = $arrFields[1];
			$arrValues = $arrValues[1];
			
			foreach($arrValues as $i => $v)
			{
				if($v == ' ')
				{
					$arrValues[$i] = '';
				}
			}
			
			
			$arrSet = array();
			foreach($arrFields as $i => $f)
			{
				// check if field exists in table
				if(!$objDatabase->fieldExists($f,$strTable))
				{
					continue;
				}
								
				$arrSet[$f] = $arrValues[$i];
			}
			
			if($strTable == 'tl_pct_customelement')
			{
				$arrImportData[$strTable][$strAlias] = $arrSet;
			}
			else
			{
				$arrImportData[$strTable][] = $arrSet;
			}
		}
		
		$this->arrImportData = $arrImportData;
		$this->blnRunImportFromTemplate = true;
		
		// kick start the regular import method
		$this->runImport();
	}
	

	/**
	 * Run import
	 */
	protected function runImport()
	{
		$this->setImportChain();
		
		if(!$this->arrImportChain || !\Contao\Input::post('import_select'))
		{
			return;
		}
		
		$objDatabase = \Contao\Database::getInstance();
		$objSession = \Contao\Session::getInstance();

		// run through import chain
		$arrImportData = array();
			
		if(!$this->blnRunImportFromTemplate)
		{
			foreach($this->arrImportChain as $table => $info)
			{
				$file = trim($GLOBALS['PCT_CUSTOMELEMENTS']['path_to_import'].'/'.$table.'.sql');
				$objFile = new \file($file);
				$data = $this->convertSqlToArray($objFile->getContent(),$info['master']);
	
				if(!is_array($data) || empty($data))
				{
					continue;
				}
	
				if($info['filter'])
				{
					$data = $this->filterImportData($data,$info['filter']);
				}
	
				$arrImportData[$table] = $data;
			}
		}
		else
		{
			$arrImportData = $this->arrImportData;
		}
		
		if(empty($arrImportData))
		{
			return;
		}
		
		
		// filter elements by reference
		foreach($arrImportData as $table => $arrData)
		{
			if(!$this->arrImportChain[$table]['reference'])
			{
				continue;
			}

			$ref = explode('.', $this->arrImportChain[$table]['reference'][1]);
			$field = $this->arrImportChain[$table]['reference'][0];
			$pTable = $ref[0];
			$pField = $ref[1];
			$pData = $arrImportData[$pTable];

			if(empty($pData))
			{
				continue;
			}

			$filterValues[$field] = array();
			foreach($pData as $row)
			{
				if(!$row[$pField])
				{
					continue;
				}
				
				// build filter values
				if(!in_array($row[$pField], $filterValues[$field]))
				{
					$filterValues[$field][] = $row[$pField];
				}
			}
			
			if(!empty($filterValues))
			{
				$filter = array
				(
					array
					(
						'field'  => $field,
						'operation' => 'IN',
						'values' => $filterValues[$field],
					),
				);
				
				$arrData = $this->filterImportData($arrData,$filter);
			}
			$arrImportData[$table] = $arrData;
		}

		if(empty($arrImportData))
		{
			return;
		}
		
		$insertId = array();
		$arrReference = array();
		$arrSession = $objSession->get($this->strSession);
		$skipImport = array();
		
		// do the import
		$objDatabase = \Contao\Database::getInstance();
		foreach($arrImportData as $table => $arrData)
		{
			if(!$objDatabase->tableExists($table))
			{
				continue;
			}
			
			$arrReference[$table] = array();
			
			$this->loadDataContainer($table);
			
			// create helper objects
			$objTable = null;
			$objDC = new \PCT\CustomElements\Helper\DataContainerHelper();
			$objDC->table = $table;
			
			if($table == 'tl_pct_customelement')
			{
				$objTable = new \PCT\CustomElements\Backend\TableCustomElement;
			}
			else if($table == 'tl_pct_customelement_group')
			{
				$objTable = new \PCT\CustomElements\Backend\TableCustomElementGroup;
			}
			else if($table == 'tl_pct_customelement_attribute')
			{
				$objTable = new \PCT\CustomElements\Backend\TableCustomElementAttribute;
			}
			
			$time = time();
			
			// build the set array
			$arrSet = array();
			foreach($arrData as $key => $row)
			{
				$title = '';
				if($table == 'tl_pct_customelement')				
				{
					$title = $row['title'];
				} 
								
				$id = $row['id'];
				$unique = array();
				// fill datacontainer object
				foreach($row as $f => $v)
				{
					$objDC->__set($f,$v);
					
					// check unique fields
					$fieldDef = $GLOBALS['TL_DCA'][$table]['fields'][$f];
					if($fieldDef['eval']['unique'])
					{
						$unique[$f] = $v;
					}
				}
				
				if(!empty($unique))
				{
					$arrWhere = array();
					foreach($unique as $f => $v)
					{
						$objCheck = $objDatabase->prepare("SELECT * FROM ".$table." WHERE ".$objDatabase->findInSet($f,$v))
								->execute();
						if($objCheck->numRows > 0)
						{
							#throw new \Exception('A unqiue field ('.$f.') with the value ('.$v. ') already exists i: '.$table);
							$strError = sprintf($GLOBALS['TL_LANG']['page_import']['error'],$title,$f,$v,$table);
							$arrErrors = $arrSession['error'];
							if(!is_array($arrErrors))
							{
								$arrErrors = array();
							}
							
							$arrErrors[] = $strError;
							$arrSession = array('error'=>array_unique($arrErrors));
							$objSession->set($this->strSession,$arrSession);
							$skipImport[] = $table;
						}
					}
				}
				
				// skip import
				if(in_array($table, $skipImport))
				{
					continue;
				}
				
				// set array
				$arrSet = $row;
				unset($arrSet['id']);
				if($row['tstamp'])
				{
					$arrSet['tstamp'] = $time;
				}
				
				//pid
				if($this->arrImportChain[$table]['reference'])
				{
					$ref = explode('.', $this->arrImportChain[$table]['reference'][1]);
					$field = $this->arrImportChain[$table]['reference'][0];
					$pTable = $ref[0];
					$pField = $ref[1];
					
					if(in_array($pTable, $skipImport))
					{
						$skipImport[] = $table;
						continue;
					}
					
					if($pField == 'id')
					{
						$arrSet['pid'] = $arrReference[$pTable][$row['pid']];
					}
				}
				
				$fieldsDb = $objDatabase->listFields($table);
				
				$fieldsDbAssoc = array();
				foreach($fieldsDb as $v)
				{
					$fieldsDbAssoc[$v['name']] = $v;
				}
				
				// check if target field is a blob field
				$arrBlobs = array();
				if(!$this->blnRunImportFromTemplate)
				{
					foreach($arrSet as $k => $v)
					{
					   $target = $fieldsDbAssoc[$k];
					   if($target['type'] == 'blob')
					   {
					   	$arrBlobs[$k] = $v;
					   	unset($arrSet[$k]);
					   }
					}
				}
				
				if(array_key_exists('pid', $arrSet) && $arrSet['pid'] === null)
				{
					$arrSet['pid'] = '';
				}
				
				$objInsert = \Contao\Database::getInstance()->prepare("INSERT INTO ".$table." %s")->set($arrSet);
				$objInsert->execute();
				
				$insert_id = $objInsert->__get('insertId');
				
				// update the blob fields for this table
				if(count($arrBlobs) > 0 && !$this->blnRunImportFromTemplate)
				{
					foreach($arrBlobs as $k => $v)
					{
						$objStatement = \Contao\Database::getInstance()->prepare("UPDATE ".$table." SET ".$k."=".$v." WHERE id=?");
						$objStatement->execute($insert_id);
					}
				}
				
				$insertId[$table] = $insert_id;
				$arrReference[$table][$id] = $insert_id;
				
				// store the name of the CustomElement imported in the session
				if($table == 'tl_pct_customelement')				
				{
					$arrSession['imported'][] = $row['title'];
				}
			}
		}
		
		$objSession->set($this->strSession,$arrSession);
	}
	
	
	/**
	 * Collect all import_ template files and return them as array
	 * @return array
	 */
	public function getImportOptionsFromTemplate()
	{
		$arrTemplates = $this->getTemplateGroup('import_');
		return $arrTemplates;
	}
	
	
	/**
	 * Return the import options as array
	 * @return array
	 */
	public function getImportOptions()
	{
		$table  = 'tl_pct_customelement';
		$objFile = new \File($GLOBALS['PCT_CUSTOMELEMENTS']['path_to_import'].'/'.$table.'.sql',false);
		if(!$objFile)
		{
			return array();
		}		
		
		$arrData = $this->convertSqlToArray($objFile->getContent(),'alias');
		if(empty($arrData))
		{
			return array();
		}
		
		$this->arrImportData = $arrData;

		$size = $this->arrPreviewSize;
		
		$arrReturn = array();
		$arrOrder = array();
		foreach($arrData as $alias => $data)
		{
			// check if a CE with that alias already exists and skip
			$objCheck = \Contao\Database::getInstance()->prepare("SELECT * FROM tl_pct_customelement WHERE alias=?")->limit(1)->execute($alias);
			if($objCheck->numRows > 0)
			{
				continue;
			}


			// images
			$labelImage = '';
			if(!empty($GLOBALS['PCT_CUSTOMELEMENTS_IMPORTS'][$alias]['images']) && is_array($GLOBALS['PCT_CUSTOMELEMENTS_IMPORTS'][$alias]['images']))
			{
				foreach($GLOBALS['PCT_CUSTOMELEMENTS_IMPORTS'][$alias]['images'] as $f)
				{
					$img = $this->getImage($f,$size[0],$size[1],$size[2]);
					$labelImage .= '<span class="image">'.\Contao\Image::getHtml($img).'</span>';
				}
			}

			$labelText = $data['title'] ? $data['title'] : $alias;
			$label = sprintf('<div class="label_container'.($labelImage ? ' hasImage':'').'">%s%s</div>',
				'<span class="label">'.$labelText.'</span>',
				($labelImage ? '<span class="images">'.$labelImage.'</span>' : '')
			);

			$arrReturn[$alias] = $label;
			$arrOrder[$alias] = $data['title'];
			
			#ksort($arrReturn);
		}
		
		// apply natural order
		natcasesort($arrOrder);
		
		// sort by order array
		$tmp = array();
		foreach($arrOrder as $k => $v)
		{
			$tmp[$k] = $arrReturn[$k];
		}
		$arrReturn = $tmp;
		unset($tmp);
		
		return $arrReturn;
	}


	/**
	 * Prepare the import data
	 * @param string
	 * @param boolean
	 * @return array
	 */
	public function convertSqlToArray($strSql,$strMasterKey='',$bolSkipEmpty=true,$arrWanted=array())
	{
		// find all information between ( and ) that not inside a string
		$preg = preg_match_all("/(\(.*?)\)/U", $strSql,$result);

		if(!$preg)
		{
			return false;
		}

		$data = $result[0];
		$rowStart = 1;

		$columns = $this->stripSqlString($data[0]);
		$columns = str_replace('`', '', $columns);
		$arrColumns = explode(',', $columns);

		if(!$strMasterKey)
		{
			$strMasterKey = 'id';
		}

		$masterKey = $strMasterKey; // identifiyer
		$masterKeyIndex = array_search($masterKey, $arrColumns);

		$arrReturn = array();
		// build values array
		for($i = $rowStart; $i < count($data); $i++)
		{
			$values = $this->stripSqlString($data[$i]);
			$arrValues = explode(',', $values);

			$key = $arrValues[$masterKeyIndex];
			$key = ltrim($key,"'");
			$key = rtrim($key,"'");

			// skip if we want a certain key only
			if(!empty($arrWanted) && !in_array($key, $arrWanted))
			{
				continue;
			}

			$arrReturn[$key] = array();
			foreach($arrValues as $k => $v)
			{
				$colname = $arrColumns[$k];

				$v = ltrim($v,"'");
				$v = rtrim($v,"'");

				if(strlen($v) < 1 && $bolSkipEmpty || $v == 'NULL' && $bolSkipEmpty)
				{
					continue;
				}

				$arrReturn[$key][$colname] = $v;
			}
		}

		return $arrReturn;
	}


	/**
	 * Filter the import data
	 * @param array
	 */
	public function filterImportData($arrData, $arrFilter)
	{
		if(!$arrData || !$arrFilter)
		{
			return array();
		}

		foreach($arrFilter as $filter)
		{
			if(!is_array($filter['values']))
			{
				$filter['values'] = array($filter['values']);
			}

			$field = $filter['field'];
			$values = $filter['values'];
			$operation = $filter['operation'];

			if(!$field || !$values || !$operation)
			{
				continue;
			}

			foreach($arrData as $index => $row)
			{
				// if the value does not exist in the haystack, continue
				if(!$row[$field])
				{
					continue;
				}

				switch($operation)
				{
					// the value must exist
				case 'IN':
					if(in_array($row[$field], $values))
					{
						continue;
					}
					unset($arrData[$index]);
					break;

					// the value must not exist
				case '!IN':
				case 'NOT IN':
					if(!in_array($row[$field], $values))
					{
						continue;
					}
					unset($arrData[$index]);
					break;

					// remove by default
				default:
					unset($arrData[$index]);
					break;
				}
			}
		}

		return $arrData;
	}

	/**
	 * Prepare sql string
	 * @param string
	 * @return string
	 */
	public function stripSqlString($strInput)
	{
		# remove brackets
		$strInput = ltrim($strInput,'(');
		$strInput = rtrim($strInput,')');

		# remove everything between /* and */
		$strInput = preg_replace("!/\*.*?\*/!ms", "", $strInput);

		# remove whitespaces after semicolons
		$strInput = preg_replace("/;\s+/m", ";", $strInput);

		# remove whitespaces after commata
		$strInput = preg_replace("/,\s+/m", ",", $strInput);

		# replace several newlines with one
		$strInput = preg_replace("/\n{2,}/m", "", $strInput);

		# Leading whitespace
		$strInput = preg_replace("/^\s*/m", "", $strInput);

		# Multiple whitespaces to one
		#$strInput = preg_replace("/ +/m", " ", $strInput);

		# Wrapping whitespace
		$strInput = trim($strInput);

		return $strInput;
	}



}