<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2014
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @subpackage	pct_customelements_plugin_customcatalog
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Backend;

/**
 * Class file
 * PageContentElementSet
 */
class PageContentElementSet extends \Contao\Backend
{
	/**
	 * DataContainer object
	 * @var object 
	 */
	protected $objDC;


	/**
	 * Initialize the controller
	 */
	public function __construct()
	{
		\Contao\System::import('BackendUser', 'User');
		parent::__construct();
		
		if(!$this->User->id)
		{
			$this->User->authenticate();
		}
		
		\Contao\System::import('Database');
		\Contao\System::import('Session');
		\Contao\System::loadLanguageFile('default');
	}


	/**
	 * Run the controller and parse the template
	 */
	public function run()
	{
		/** @var \Contao\BackendTemplate|object $objTemplate */
		$this->Template = new \Contao\BackendTemplate('be_page_contentelementset');
		$this->Template->main = '';
		
		$strTable = \Contao\Input::get('table');
		$strField = \Contao\Input::get('field');
		$strKey = \Contao\Input::get('key');
		
		// @var object Session
		$objSession = \Contao\Session::getInstance();
		if(version_compare(VERSION, '4','>='))
		{
			$objSession = \Contao\System::getContainer()->get('session');
		}
		$arrSession = $objSession->get('contentelementsets');
		if(!is_array($arrSession['categories']))
		{
			$arrSession['categories'] = array();
		}
		
		// Define the current ID
		define('CURRENT_ID', (\Contao\Input::get('table') ? $this->Session->get('CURRENT_ID') : \Contao\Input::get('id')));

		if(!$GLOBALS['TL_DCA'][$strTable]['config']['dataContainer'])
		{
			$this->loadDataContainer($strTable);
		}
		
		$this->objDC = new \PCT\CustomElements\Helper\DataContainerHelper();
		$this->objDC->table = $strTable;
		$this->objDC->field = $strField;
		$this->objDC->id = \Contao\Input::get('id');
		
		// Set the active record
		if ($this->Database->tableExists($strTable))
		{
			/** @var \Model $strModel $strModel */
			$strModel = \Contao\Model::getClassFromTable($strTable);

			if (class_exists($strModel))
			{
				$objModel = $strModel::findByPk(\Contao\Input::get('id'));

				if ($objModel !== null)
				{
					$this->objDC->activeRecord = $objModel;
				}
			}
		}
		
		// @var object
		$objContentElementSet = new \PCT\CustomElements\Core\ContentElementSet;
		
		$arrElementSets = $objContentElementSet->getContentElementSets();
		
		$arrElements = array();
		$arrCategories = array('custom');
		$arrSkipped = array();
		$i = 0;
		foreach($arrElementSets as $k => $v)
		{
			$category = $v['category'];
			
			if(strlen($category) > 0)
			{
				$arrCategories[] = $category;
			}
			// default category
			else
			{
				$category = 'custom';
			}
			
			// filter by category
			if(!empty($arrSession['categories']))
			{
				if(!in_array($category,$arrSession['categories']))
				{
					continue;
				}
			}
			
			$thumb = strlen($v['thumb']) > 0 ? $v['thumb'] : PCT_CUSTOMELEMENTS_PATH.'/assets/img/contentelementset.jpg';
			$img = '';
			if(file_exists(TL_ROOT.'/'.$thumb))
			{
				$img = \Contao\Image::getHtml($thumb ,$v['label']);
			}
			
			// skip when there is no image
			if(strlen($img) < 1)
			{
				if(\Config::get('debugMode'))
				{
					$arrSkipped[] = $k;
				}
			}
			
			$arrClass = array('item',$k);
			if($i == 0) {$arrClass[] = 'first';}
			if($i >= count($arrElementSets) -1 ) {$arrClass[] = 'last';}
			$i%2 == 0 ? $arrClass[] = 'even' : $arrClass[] = 'odd';
			
			// collect output
			$element = array
			(
				'label'	=> $v['label'] ?: $k,
				'id' 	=> $k,
				'src'	=> $thumb,
				'img'	=> $img,
				'name'	=> $k,
				'class'	=> implode(' ', $arrClass),
			);
			
			// collect ordered by category
			$arrElements[$category]['label'] = $GLOBALS['TL_LANG']['CONTENTELEMENTSETS']['CATEGORIES'][$category] ?: $category;
			$arrElements[$category]['elements'][$k] = $element;
			
			unset($element);
			
			$i++;
		}
		
		// reorder in cols
		$intColumns = 2;
		$arrColumns = array();
		$arrProcessed = array();
		
		foreach($arrElements as $category => $data)
		{
			// build columns
			$index = 0;
			for($col = 0; $col < $intColumns; $col++)
			{
				$i = 0;
				foreach($data['elements'] as $element)
				{
					if($i%$intColumns == $col)
					{
						$arrColumns[$category][$col][$index] = $element;
						$index++;
					}
					$i++;
				}
			}
		}
		$this->Template->columns = $arrColumns;
		$this->Template->columnCount = $intColumns;
		
		// stort arrays
		$arrCategories = array_filter(array_unique($arrCategories));
		asort($arrCategories);
		
		// store filter
		if(\Contao\Input::post('FORM_SUBMIT') == 'tl_filter')
		{
			$arrSession['categories'] = \Contao\Input::post('categories');
			// set session
			$objSession->set('contentelementsets',$arrSession);
			// reload
			$this->reload();
		}
		
		// category filter select
		$arrField = array
		(
			'inputType'	=> 'select',
			'label'		=> &$GLOBALS['TL_LANG']['contentelementsets']['categories'],
			'options'	=> array_values($arrCategories),
			'reference'	=> $GLOBALS['TL_LANG']['CONTENTELEMENTSETS']['CATEGORIES'],
			'eval'		=> array('includeBlankOption'=>true,'chosen'=>true,'multiple'=>true),
		);
		
		$strClass = $GLOBALS['BE_FFL'][$arrField['inputType']];
		$arrAttributes = $strClass::getAttributesFromDca($arrField,'categories',$arrSession['categories'] ?: array());
		$objWidget = new $strClass($arrAttributes);
		$this->Template->categoryWidget = $objWidget->parse();
		
		// back button
		$strReferer = \Contao\Controller::addToUrl('act=paste&mode=create&rkey=contentelementset&rt='.REQUEST_TOKEN,true,array('key'));
		
		$this->Template->back = '<a class="header_back" onclick="Backend.getScrollOffset()" accesskey="b" title="'.\specialchars($GLOBALS['TL_LANG']['MSC']['backBTTitle']).'" href="'.$strReferer.'">'.$GLOBALS['TL_LANG']['MSC']['backBT'].'</a>';
		
		$this->Template->main = '';
		$this->Template->action = \ampersand(\Contao\Environment::get('request'));
		$this->Template->objDataContainer = $this->objDC;
		$this->Template->elements = $arrElements;
		
		// import
		if(\Contao\Input::post('install') != '' && \Contao\Input::post('ELEMENTSET') != '')
		{
			$blnSuccess = $objContentElementSet->importByName(\Contao\Input::post('ELEMENTSET'));
			if($blnSuccess)
			{
				// clear the clipboard
				$arrClipboard = $objSession->get('CLIPBOARD');
				unset($arrClipboard[$strTable]);
				$objSession->set('CLIPBOARD',$arrClipboard);
				
				// back to list view
				$strReferer = \Contao\Controller::addToUrl('rt='.REQUEST_TOKEN.'&clipboard=1',true,array('key','rkey'));
				
				\Contao\Controller::redirect($strReferer);
			}
			
			
		}
		
		return $this->Template->parse();
	}
}