<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Backend;

/**
 * Imports
 */
use PCT\CustomElements\Core\PluginFactory as PluginFactory;
use PCT\CustomElements\Helper\DcaHelper as DcaHelper;

/**
 * Class file
 * TableCustomElement
 */
class TableCustomElementPlugin extends \Contao\Backend
{
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser', 'User');
	}
	
	
	/**
	 * Modify the backend panel
	 * @param object
	 */
	public function modifyPalette($objDC)
	{
		#$objActiveRecord = \Contao\Database::getInstance()->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->execute($objDC->id);
	}


	/**
	 * Return all registered plugins as array
	 * @param DataContainer
	 * @return array
	 */
	public function getPlugins($objDC)
	{
		$arrPlugins = PluginFactory::getRegistered();
		if(count($arrPlugins) < 1)
		{
			return array();
		}
		
		$arrReturn = array();
		foreach($arrPlugins as $v)
		{
			$arrReturn[$v] = $GLOBALS['TL_LANG']['PCT_CUSTOMELEMENTS']['PLUGINS'][$v][0] ? $GLOBALS['TL_LANG']['PCT_CUSTOMELEMENTS']['PLUGINS'][$v][0] .' ['.$v.']' : $v;
		}
		
		return $arrReturn;
	}
	
	
	/**
	 * Return the plugin description
	 * @param object
	 */
	public function getDescription($objDC)
	{
		$this->loadLanguageFile('default');
		
		$objActiveRecord = $objDC->activeRecord;
		$strModel = \Contao\Model::getClassFromTable($objDC->table);
		if($objActiveRecord === null && class_exists($strModel))
		{
			$objActiveRecord = $strModel::findByPk($objDC->id);
		}
		else
		{
			$objActiveRecord = \Contao\Database::getInstance()->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		}
		
		if(!$objActiveRecord->plugin)
		{
			return '';
		}
		
		$strBuffer = $GLOBALS['TL_LANG']['PCT_CUSTOMELEMENTS']['PLUGINS'][$objActiveRecord->plugin][1];
		
		return '<div class="field backend_explanation widget">'.$strBuffer.'</div>';
	}
	

	/**
	 * Generate label
	 * @param array
	 * @return string
	 */
	public function getLabel($arrRow)
	{
		$strLabel = $GLOBALS['TL_LANG']['PCT_CUSTOMELEMENTS']['PLUGINS'][$arrRow['plugin']][0] ? $GLOBALS['TL_LANG']['PCT_CUSTOMELEMENTS']['PLUGINS'][$arrRow['plugin']][0] .' ['.$arrRow['plugin'].']' : $arrRow['plugin'];
		
		return $strLabel;
	}
	
	
	/**
	 * Check available plugins
	 * @param object
	 */
	public function checkPluginsAvailable($objDC)
	{
		// close table if there are no plugins registered
		if(empty($GLOBALS['PCT_CUSTOMELEMENTS']['PLUGINS']) || !is_array($GLOBALS['PCT_CUSTOMELEMENTS']['PLUGINS']))
		{
			$GLOBALS['TL_DCA'][$objDC->table]['config']['closed'] = true;
		}
		
		// check plugins available
		$arrPlugins = PluginFactory::getAvailable();
		if(count($arrPlugins) < 1)
		{
			$GLOBALS['TL_DCA'][$objDC->table]['config']['closed'] = true;
		} 
	}
	
	
	/**
	 * Check CustomElement Version and show update alert is nessessary
	 * @param object
	 */	
	public function checkPluginRequirements($objDC)
	{
		if($_POST && \Contao\Input::get('act') != 'edit')
		{
			return;
		}
		
		$objActiveRecord = $objDC->activeRecord;
		$strModel = \Contao\Model::getClassFromTable($objDC->table);
		if($objActiveRecord === null && class_exists($strModel))
		{
			$objActiveRecord = $strModel::findByPk($objDC->id);
		}
		else
		{
			$objActiveRecord = \Contao\Database::getInstance()->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		}
		
		if(!$objActiveRecord->plugin || !$GLOBALS['PCT_CUSTOMELEMENTS']['PLUGINS'][$objActiveRecord->plugin]['requires'])
		{
			return '';
		}
		
		foreach($GLOBALS['PCT_CUSTOMELEMENTS']['PLUGINS'][$objActiveRecord->plugin]['requires'] as $mod => $req)
		{
			switch($mod)
			{
				case 'pct_customelements':
					if(version_compare(PCT_CUSTOMELEMENTS_VERSION,$req,'<'))
					{
						\Message::addInfo(sprintf($GLOBALS['TL_LANG']['PCT_CUSTOMELEMENTS']['MSC']['pluginRequirementInfo'][$mod], $objActiveRecord->plugin, PCT_CUSTOMCATALOG_REQ_CE));
					}
					break;
			}
		}
		
	}
	
	
	/**
	 * Check permissions
	 */
	public function checkPermission()
	{
		if ($this->User->isAdmin)
		{
			return;
		}
		
		// Set root IDs
		if (!is_array($this->User->pct_customelements) || empty($this->User->pct_customelements))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->pct_customelements;
		}
		
		$GLOBALS['TL_DCA']['tl_pct_customelement']['list']['sorting']['root'] = $root;
		
		// Check permissions to add new elements
		if (!$this->User->hasAccess('create', 'pct_customelementsp'))
		{
			$GLOBALS['TL_DCA']['tl_pct_customelement']['config']['closed'] = true;
		}
		
		// Check current action
		switch (\Contao\Input::get('act'))
		{
			case 'create':
			case 'select':
			case 'show':
				// Allow
				break;
			case 'edit':
				// Dynamically add the record to the user profile
				if (!in_array(\Contao\Input::get('id'), $root))
				{
					$arrNew = $this->Session->get('new_records');
	
					if (is_array($arrNew['tl_pct_customelement']) && in_array(\Contao\Input::get('id'), $arrNew['tl_pct_customelement']))
					{
						// Add permissions on user level
						if ($this->User->inherit == 'custom' || !$this->User->groups[0])
						{
							$objUser = $this->Database->prepare("SELECT pct_customelements, pct_customelementsp FROM tl_user WHERE id=?")
								->limit(1)
								->execute($this->User->id);
	
							$arrPermissions = \deserialize($objUser->pct_customelementsp);
	
							if (is_array($arrPermission) && in_array('create', $arrPermissions))
							{
								$arrElements = \deserialize($objUser->pct_customelements);
								$arrElements[] = \Contao\Input::get('id');
	
								$this->Database->prepare("UPDATE tl_user SET pct_customelements=? WHERE id=?")
								->execute(serialize($arrElements), $this->User->id);
							}
						}
						// Add permissions on group level
						elseif ($this->User->groups[0] > 0)
						{
							$objGroup = $this->Database->prepare("SELECT pct_customelements, pct_customelementsp FROM tl_user_group WHERE id=?")
							->limit(1)
							->execute($this->User->groups[0]);
	
							$arrPermissions = \deserialize($objGroup->pct_customelementsp);
	
							if (is_array($arrPermissions) && in_array('create', $arrPermissions))
							{
								$arrElements = \deserialize($objGroup->pct_customelements);
								$arrElements[] = \Contao\Input::get('id');
	
								$this->Database->prepare("UPDATE tl_user_group SET pct_customelements=? WHERE id=?")
								->execute(serialize($arrElements), $this->User->groups[0]);
							}
						}
	
						// Add new element to the user object
						$root[] = \Contao\Input::get('id');
						$this->User->pct_customelements = $root;
					}
					break;
				}
	
			// No break;
			case 'delete':
				if (!$this->User->hasAccess('delete', 'pct_customelementsp'))
				{
					$this->log('Not enough permissions to '.\Contao\Input::get('act').' slider ID "'.\Contao\Input::get('id').'"', 'tl_pct_pct_customelement checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;

			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
				$session = $this->Session->getData();
				if (\Contao\Input::get('act') == 'deleteAll' && !$this->User->hasAccess('delete', 'pct_customelementsp'))
				{
					$session['CURRENT']['IDS'] = array();
				}
				else
				{
					$session['CURRENT']['IDS'] = array_intersect($session['CURRENT']['IDS'], $root);
				}
				$this->Session->setData($session);
				break;
	
			default:
				if (strlen(\Contao\Input::get('act')))
				{
					$this->log('Not enough permissions to '.\Contao\Input::get('act').' element ID "'.\Contao\Input::get('id').'"', 'tl_pct_pct_customelement checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;
		}
	}


	/**
	 * Return the edit button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function editButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('edit', 'pct_customelement_pluginp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.\specialchars($title).'"'.$attributes.'>'.\Contao\Image::getHtml($icon, $label).'</a> ' : \Contao\Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}
	
	/**
	 * Return the delete button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function deleteButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('delete', 'pct_customelement_pluginp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.\specialchars($title).'"'.$attributes.'>'.\Contao\Image::getHtml($icon, $label).'</a> ' : \Contao\Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}
	
	/**
	 * Return the back button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function backButton($row, $href, $label, $title, $attributes, $table)
	{
		return '<a class="header_back" href="'.$this->getReferer(true).'" '.$attributes.'>'.$label.'</a>';
	}

	/**
	 * Return the "toggle visibility" button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function toggleIcon($row, $href, $label, $title, $icon, $attributes)
	{
		$objInput = \Contao\Input::getInstance();
		if (strlen($objInput->get('tid')))
		{
			$this->toggleVisibility($objInput->get('tid'), ($objInput->get('state') == 1));
			$this->redirect($this->getReferer());
		}

		// Check permissions AFTER checking the tid, so hacking attempts are logged
		if (!$this->User->isAdmin && !$this->User->hasAccess('tl_pct_customelement_plugin::published', 'alexf'))
		{
			return '';
		}

		$href .= '&amp;tid='.$row['id'].'&amp;state='.($row['published'] ? '' : 1);

		if (!$row['published'])
		{
			$icon = 'invisible.gif';
		}
		
		return '<a href="'.$this->addToUrl($href).'" title="'.\specialchars($title).'"'.$attributes.'>'.\Contao\Image::getHtml($icon, $label, 'data-state="' . (!$row['published'] ? 0 : 1) . '"').'</a> ';
	}
	
	/**
	 * Return the "toggle visibility" button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function toggleButton($row, $href, $label, $title, $icon, $attributes)
	{
		$objInput = \Contao\Input::getInstance();
		if (strlen($objInput->get('tid')))
		{
			$this->toggleVisibility($objInput->get('tid'), ($objInput->get('state') == 1));
			$this->redirect($this->getReferer());
		}

		// Check permissions AFTER checking the tid, so hacking attempts are logged
		if (!$this->User->isAdmin && !$this->User->hasAccess('tl_pct_customelement_plugin::published', 'alexf'))
		{
			return '';
		}

		$href .= '&amp;tid='.$row['id'].'&amp;state='.($row['published'] ? '' : 1);

		if (!$row['published'])
		{
			$icon = 'invisible.gif';
		}
		
		// set the href directly
		return '<a href="'.$href.'" title="'.\specialchars($title).'"'.$attributes.'>'.\Contao\Image::getHtml($icon, $label).'</a> ';
	}
	
	/**
	 * Disable/enable a user group
	 * @param integer
	 * @param boolean
	 */
	public function toggleVisibility($intId, $blnVisible)
	{
		// Check permissions to edit
		$objInput = \Contao\Input::getInstance();
		$objInput->setGet('id', $intId);
		$objInput->setGet('act', 'toggle');
		
		$objDcaHelper = new DcaHelper();
		$objDcaHelper->setUpdateFlag();
		
		// Check permissions to publish
		if (!$this->User->isAdmin && !$this->User->hasAccess('tl_pct_customelement_plugin::published', 'alexf'))
		{
		   $this->log('Not enough permissions to publish/unpublish item ID "'.$intId.'"', 'tl_pct_customelement_attribute toggleVisibility', TL_ERROR);
		   $this->redirect('contao/main.php?act=error');
		}

		$objVersions = new \Versions('tl_pct_customelement_attribute', $intId);
		$objVersions->initialize();
		
		// Trigger the save_callback
		if (is_array($GLOBALS['TL_DCA']['tl_pct_customelement_plugin']['fields']['published']['save_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_pct_customelement_plugin']['fields']['published']['save_callback'] as $callback)
			{
				$blnVisible = \Contao\System::importStatic($callback[0])->{$callback[1]}($blnVisible, $this);
			}
		}
		
		// Update the database
		\Contao\Database::getInstance()->prepare("UPDATE tl_pct_customelement_plugin SET tstamp=". time() .", published='" . ($blnVisible ? 1 : '') . "' WHERE id=?")->execute($intId);

		$objVersions->create();
		
		$this->log('A new version of record "tl_pct_customelement_plugin.id='.$intId.'" has been created', 'tl_pct_customelement_plugin toggleVisibility()', TL_GENERAL);
	}
	
}