<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Core;

/**
 * Imports
 */
use \PCT\CustomElements\Core\Cache as Cache;

/**
 * Class file
 * ContentElementSet
 */
class ContentElementSet extends \Contao\Controller
{
	/**
	 * Find all content element set templates registered or in templates folder
	 * @return array
	 */
	public function getContentElementSets($blnCached=true)
	{
		// look up from cache
		if($blnCached && Cache::getContentElementSets() !== null)
		{
			return Cache::getContentElementSets();
		}
		
		$arrReturn = array();
		
		foreach($this->getTemplateGroup($GLOBALS['PCT_CUSTOMELEMENTS']['contentElementSetTemplatePrefix']) as $k => $name)
		{
			// includes $arrImport
			include \Contao\Controller::getTemplate($k,'html5');
			
			if(!isset($arrImport) || !is_array($arrImport))
			{
				continue;
			}
			
			$arrReturn[$k] = $arrImport;
		}
		
		// add to cache
		if($blnCached)
		{
			Cache::addContentElementSets($arrReturn);	
		}
		
		return $arrReturn;
	}
	
	
	/**
	 * Return all categories as array
	 * @return array
	 */
	public function getCategories()
	{
		$arrSets = $this->getContentElementSets();
		
		$arrReturn = array('custom');
		foreach($arrSets as $k => $v)
		{
			$category = $v['category'];
			
			if(strlen($category) > 0)
			{
				$arrReturn[] = $category;
			}
		}
		
		return $arrReturn;
	}
	
	
	/**
	 * Export the content elements by their ids and return the path to the template file
	 * @param array		Ids of the content elements
	 * @param array		Additional parameters to pass to the file
	 * @return object	The files object of the created file
	 */
	public function exportByIds($arrIds,$arrParams)
	{
		if(!is_array($arrIds) || empty($arrIds))
		{
			return null;
		}
		
		$objContentModels = \ContentModel::findMultipleByIds( $arrIds,array('order'=>'sorting') );
		if($objContentModels === null)
		{
			return null;
		}
		
		$strTable = 'tl_content';
		
		// find all installed content element CustomElements
		$arrCustomElements = \PCT\CustomElements\Core\CustomElements::getInstance()->getCustomElements($strTable);
		
		$arrExport = array();
		foreach($objContentModels as $objModel)
		{
			$arrRow = $objModel->row();
			
			$arrExport['tl_content'][] = $arrRow;
			
			// tl_pct_customelement_vault
			if(in_array($objModel->type,$arrCustomElements))
			{
				$objVaultModels = \PCT\CustomElements\Models\VaultModel::findByPidAndSource($arrRow['id'],'tl_content');
				if($objVaultModels !== null)
				{
					foreach($objVaultModels as $objVault)
					{
						$arrExport['tl_pct_customelement_vault'][] = $objVault->row();
					}
				}
			}
		}
		
		$strName = $arrParams['name'] ?: 'myset_'.\Contao\Input::get('id');
		
		$strContent = $this->prepareFileContent($arrExport,$arrParams);
		if(strlen($strContent) < 1)
		{
			return null;
		}
		
		$strFile = 'templates/'.$GLOBALS['PCT_CUSTOMELEMENTS']['contentElementSetTemplatePrefix'].$strName.'.html5';
		$objFile = $this->writeFile($strFile, $strContent);
		
		return $objFile;
	}
	
	
	/**
	 * Prepare the file content from a structured table record array
	 * @param array|null
	 * @param array			Additional content
	 * @return string
	 */
	public function prepareFileContent($arrData,$arrParams=array())
	{
		if(count($arrData) < 1)
		{
			return '';
		}
		
		$strReturn = '';

$arrParts = array();

//-- head
$strHead = 
"<?php
/**
 * Hello, I'm a the content element set template file: ".$strName."
 */

";

$arrInfo = array('tstamp' => \Contao\System::parseDate('d.m.Y H:i',time()));
if(count($arrParams) > 0)
{
	foreach($arrParams as $k => $v)
	{
		$arrInfo[$k] = $v;
	}
}

		$arrInserts = array();
		
		foreach($arrData as $table => $records)
		{
			foreach($records as $row)
			{
				$fields = array_keys($row);
				$values = array_values($row);
				
				// convert binary
				foreach($values as $k => $v)
				{
					$field = $fields[$k];
					
					if(empty($v))
					{
						unset($fields[$k]);
						unset($values[$k]);
						continue;
					}
					
					if(is_array($v))
					{
						$v = serialize($v);
					}
					
					if(\Contao\Validator::isBinaryUuid($v))
					{
						$uuid = \Contao\StringUtil::binToUuid($v);
						
						$objFile = \FilesModel::findByUuid($v);
						if($objFile)
						{
							$arrInserts['tl_files'][$uuid] = $objFile->path;
						}
						
						
						$values[$k] = $uuid;
					}
				
					// binary in multiSRC fields
					if(in_array($field, $GLOBALS['PCT_CUSTOMELEMENTS']['multiSRC_fields']) && !empty($row[$field]))
					{
						$arrValue = \deserialize($row[$field]);
						if(is_array($arrValue) && !empty($arrValue))
						{
							$values[$k] = serialize( array_map('StringUtil::binToUuid', $arrValue) );
						
							foreach($arrValue as $v)
							{
								$uuid = \Contao\StringUtil::binToUuid($v);
								
								$objFile = \FilesModel::findByUuid($v);
								if($objFile)
								{
									$arrInserts['tl_files'][$uuid] = $objFile->path;
								}
							}
						}
					}
				}
				
				$arrInserts[] = array
				(
					'table'		=> $table,
					'fields'	=> $fields,
					'values'	=> $values,
					#'sql'		=>$this->buildInsert($table,$fields,$values)
				);
			}
		}
		
		if(!empty($arrData['tl_pct_customelement_vault']) && is_array($arrData['tl_pct_customelement_vault']))
		{
			foreach($arrData['tl_pct_customelement_vault'] as $row)
			{
				$arrWizard = \deserialize($row['data_blob']);
				if(empty($arrWizard) || !is_array($arrWizard))
				{
					continue;
				}
				
				if(empty($arrWizard['values']) || !is_array($arrWizard['values']))
				{
					continue;
				}
				
				foreach($arrWizard['values'] as $field => $v)
		   		{
		      		if(empty($v))
		      		{
			      		continue;
		      		}
		      		
		      		$v = \deserialize($v);
		      		
		      		if(!is_array($v))
		      		{
			      		if(\Contao\Validator::isBinaryUuid($v))
				  		{
				   			$uuid = \Contao\StringUtil::binToUuid($v);
				   			
				   			$objFile = \FilesModel::findByUuid($v);
				   			if($objFile)
				   			{
				   				$arrInsert['tl_files'][$uuid] = $objFile->path;
							}
				   		}
				   		else if(\Contao\Validator::isStringUuid($v))
				   		{
					   		$uuid = $v;
					   		$bin = \Contao\StringUtil::binToUuid($v);
					   		$objFile = \FilesModel::findByUuid($uuid);
				   			if($objFile)
				   			{
				   				$arrInserts['tl_files'][$uuid] = $objFile->path;
				   			}
				   		}
				   		
			   		}
			   		
			   		// binary in multiSRC fields
			   		else if(in_array($field, $GLOBALS['PCT_CUSTOMELEMENTS']['multiSRC_fields']) && is_array($v))
			   		{
			   			$arrValue = \deserialize($v);
			   			if(is_array($arrValue) && !empty($arrValue))
			   			{
			   				foreach($arrValue as $v)
			   				{
			   					$uuid = $v;
						   		$bin = \Contao\StringUtil::binToUuid($v);
						   		$objFile = \FilesModel::findByUuid($bin);
					   			if($objFile)
					   			{
					   				$arrInserts['tl_files'][$uuid] = $objFile->path;
					   			}
			   				}
			   			}
			   		}
				}
			}
		}
		
	
		// build string
		$arrParts[0] = $strHead;
		$arrParts[1] = '$arrImport = '.var_export($arrInfo,true).';
		';
		
		$arrParts[2] = '$arrImport["sql"] = '.strval(var_export($arrInserts, true)).';';
		#$arrParts[2] = '$arrImport["sql"] = '.serialize($arrInserts).';';
		
		return implode('', $arrParts);
	}
	
	
	/**
	 * Write the template file
	 * @param string
	 * @param string
	 * @return boolean
	 */
	public function writeFile($strFile,$strContent)
	{
		$objFile = new \File($strFile);
		$strContent = str_replace( array("\t","\r"), "", $strContent);
		$strContent = preg_replace(array('/^[\t ]+/m', '/[\t ]+$/m', '/\n\n+/'), array('', '', "\n"), $strContent);
		
		#$objMinify = new MatthiasMullie\Minify\JS();
		#$objMinify->add($strContent);
		#$strContent = $objMinify->minify();
		
		if(!$objFile->exists())
		{
			\File::putContent($strFile,$strContent);
		}
		else
		{
			$objFile->write($strContent);
		}
		
		return $objFile;
	}
	
	
	/**
	 * Import / insert the content element set
	 * @param string
	 * @param integer
	 * @return boolean
	 */
	public function importByName($strName)
	{
		if(TL_MODE == 'FE' || strlen($strName) < 1)
		{
			return false;
		}
		
		$strFile = \Contao\Controller::getTemplate($strName,'html5');
		
		if(!file_exists($strFile))
		{
			return false;
		}
		
		
		// includes $arrImport
		include \Contao\Controller::getTemplate($strName,'html5');
		
		if(empty($arrImport['sql']))
		{
			return false;
		}
		
		$intIncrementSorting = 64;
		
		// @var object Database
		$objDatabase = \Contao\Database::getInstance();
		
		$objPasteAfter = null;
		
		if(\Contao\Input::get('mode') == 1)
		{
			$objPasteAfter = \ContentModel::findByPk(\Contao\Input::get('pid'));
		}
		else if(\Contao\Input::get('mode') == 2 && \Contao\Input::get('table') == 'tl_content') // root
		{
			$ptable = $GLOBALS['TL_DCA'][\Contao\Input::get('table')]['config']['ptable'];
			
			// find lowest sorting
			$objResult = \Contao\Database::getInstance()->prepare("SELECT * FROM ".\Contao\Input::get('table')." WHERE pid=? AND ptable=? ORDER BY sorting")->limit(1)->execute(\Contao\Input::get('pid'),$ptable);
			
			$sorting = ($objResult->sorting > 0 ? $objResult->sorting - 16 : 0);
			if($sorting < 0)
			{
				$sorting = 0;
			}
			
			// build psydo paste after element
			$objPasteAfter = new \StdClass;
			$objPasteAfter->pid = \Contao\Input::get('pid');
			$objPasteAfter->sorting = $sorting;
			$objPasteAfter->ptable = $ptable;
		}
		
		if($objPasteAfter === null)
		{
			return false;
		}
		
		$this->arrTl_files = null;
		if(is_array($arrImport['sql']['tl_files']))
		{
			$this->arrTl_files = $arrImport['sql']['tl_files'];
			unset($arrImport['sql']['tl_files']);
		}
		
		$arrData = array();
		foreach($arrImport['sql'] as $data)
		{
			$strTable = $data['table'];
		
			if(!$objDatabase->tableExists($strTable))
			{
				\Contao\System::log('Table '.$strTable.' does not exist!',__METHOD__,TL_ERROR);
				
				// throw error
				throw new \Exception('Table '.$strTable.' does not exist!');
			}
			
			$row = array_combine($data['fields'], $data['values']);
			foreach($row as $field => $value)
			{
				if(!$objDatabase->fieldExists($field,$strTable))
				{
					unset($row[$field]);
				}
			}
			
			$arrData[$strTable][] = $row;
		}
		
		$time = time();
		
		$arrVaultData = $arrData['tl_pct_customelement_vault'];
		unset($arrData['tl_pct_customelement_vault']);
		
		// insert a wrapper start at first position
		$arrWrapper = array
		(
			'type' 		=> 'pct_contentelementset_start',
			'pid'  		=> $objPasteAfter->pid,
			'ptable'	=> $objPasteAfter->ptable,
			'tstamp'	=> $time,
			'alt'		=> $strName,
		);
		\array_insert($arrData['tl_content'],0,array($arrWrapper));
		
		// insert a wrapper stop at last position
		$arrWrapper = array
		(
			'type' 		=> 'pct_contentelementset_stop',
			'pid'  		=> $objPasteAfter->pid,
			'ptable'	=> $objPasteAfter->ptable,
			'tstamp'	=> $time,
			'alt'		=> $strName,
		);
		\array_insert($arrData['tl_content'],count($arrData['tl_content']),array($arrWrapper));
		
		$arrPids = array();
		$arrNewRecords = array();
		$arrMoveRecords = array();
		
		// prepare the set array
		$arrSorting = array('tl_content'=>(int)$objPasteAfter->sorting);
		
		foreach($arrData as $strTable => $arrRows)
		{
			$hasSorting = $objDatabase->fieldExists('sorting',$strTable);
			
			// check if there is an element at the same position
			if($hasSorting)
			{
				$objElementBelow = $objDatabase->prepare
				(
					"SELECT * FROM ".$strTable." WHERE sorting>?".
					((int)$objPasteAfter->pid > 0 ? " AND pid=?" : "").
					(strlen($objPasteAfter->ptable) > 0 ? " AND ptable=?" : "").
					" ORDER BY sorting"
				)->execute($objPasteAfter->sorting,$objPasteAfter->pid,$objPasteAfter->ptable);
				
				if($objElementBelow->numRows > 0)
				{
					$arrMoveRecords[$strTable] = array_merge($arrMoveRecords[$strTable] ?: array(),$objElementBelow->fetchEach('id'));
				}
			}
			
			foreach($arrRows as $i => $arrRow)
			{
				if($hasSorting)
				{
					// sorting
					$sorting = $arrSorting[$strTable] + $intIncrementSorting;
					
					$arrRow['sorting'] = $sorting;
										
					// remember sorting
					$arrSorting[$strTable] = $sorting;
				}
				
				// update
				$arrRow['pid'] = $objPasteAfter->pid;
				
				if(isset($arrRow['tstamp']))
				{
					$arrRow['tstamp'] = $time;
				}
				
				if(strlen($objPasteAfter->ptable) && $objDatabase->fieldExists('ptable',$strTable))
				{
					$arrRow['ptable'] = $objPasteAfter->ptable;
				}
				
				$arrData[$strTable][$i] = $arrRow;
			}
		}
		
		// make a key sort to sort tl_content before tl_pct_...
		ksort($arrData);
		
		// do the insert
		foreach($arrData as $strTable => $arrRows)
		{
			$hasSorting = $objDatabase->fieldExists('sorting',$strTable);
			
			foreach($arrRows as $i => $row)
			{
				$intOldId = $row['id'];
				unset($row['id']);
				
				// check fields and convert binary data
				foreach($row as $f => $v)
				{
					if(!$objDatabase->fieldExists($f,$strTable))
					{
						unset($row[$i][$f]);
						continue;
					}
					
					// check if a value is a uuid string
					if(\Contao\Validator::isStringUuid($v))
					{
						$uuid = $v;
						
						$bin = null;
						if($this->arrTl_files[$uuid])
						{
							$strFile = $this->arrTl_files[$uuid];
							if(file_exists(TL_ROOT.'/'.$strFile))
							{
								$objFile = \Dbafs::addResource($strFile);
								if($objFile)
								{
									$bin = $objFile->uuid;
								}
							}
						}
						else
						{
							$bin = \Contao\StringUtil::uuidToBin($uuid);
						}
										
						$v = $bin;
						
						$row[$f] = $v;
					}
						
					// binary in multiSRC fields
					if(in_array($f, $GLOBALS['PCT_CUSTOMELEMENTS']['multiSRC_fields']) && !empty($v))
					{
						$arrValue = \deserialize($v);
						if(is_array($arrValue) && count($arrValue) > 0)
						{
							$tmp = array();
							foreach($arrValue as $uuid)
							{
								$bin = null;
								
								if($this->arrTl_files[$uuid])
								{
									$strFile = $this->arrTl_files[$uuid];
									if(file_exists(TL_ROOT.'/'.$strFile))
									{
										$objFile = \Dbafs::addResource($strFile);
										if($objFile)
										{
											$bin = $objFile->uuid;
										}
									}
								}
								else
								{
									$bin = \Contao\StringUtil::uuidToBin($uuid);
								}
								$tmp[] = $bin;
							}
							
							$v = serialize($tmp);
							unset($tmp);
							
							$row[$f] = $v;
						}
					}
				}
				
				$objInsert = $objDatabase->prepare("INSERT INTO ".$strTable." %s")->set($row)->execute();
				
				$_insertId = $objInsert->__get('insertId');
				
				if($intOldId > 0)
				{
					$arrPids[$strTable][$intOldId] = $_insertId;
				}
				
				$arrNewRecords[$strTable][] = $_insertId;
			}
			
			// move records
			if(is_array($arrMoveRecords[$strTable]))
			{
				$arrMoveRecords[$strTable] = array_filter(array_unique($arrMoveRecords[$strTable]));
			}
			
			if(!empty($arrMoveRecords[$strTable]))
			{
				$objRecords = $objDatabase->prepare("SELECT * FROM ".$strTable." WHERE id IN(".implode(',', $arrMoveRecords[$strTable]).") ORDER BY sorting")->execute();
				while($objRecords->next())
				{
					$sorting = $arrSorting[$strTable] + $intIncrementSorting;
				
					$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set(array('sorting'=>$sorting))->execute($objRecords->id);
					
					$arrSorting[$strTable] = $sorting;	
				}
			}
		}
		
		// tl_pct_customelement_vault
		if(is_array($arrVaultData) && !empty($arrVaultData))
		{
			foreach($arrVaultData as $row)
			{
				$strTable = 'tl_pct_customelement_vault';
				
				$source = $row['source'];
				if(strlen($source) < 1 || !$objDatabase->tableExists($source))
				{
					continue;
				}
				
				$oldId = $row['id'];
				unset($row['id']);
				
				$intPid = 0;
				if($arrPids[$source][$row['pid']] > 0)
				{
					$intPid = $arrPids[$source][$row['pid']];
				}
				
				$row['tstamp'] = $time;
				$row['pid'] = $intPid;
				
				// binary data
				if($row['type'] == 'wizard')
				{
					$arrWizard = \deserialize($row['data_blob']);
					
					if(empty($arrWizard) || !is_array($arrWizard))
					{
						continue;
					}
					
					if(empty($arrWizard['values']) || !is_array($arrWizard['values']))
					{
						continue;
					}
					
					foreach($arrWizard['values'] as $field => $v)
			   		{
			      		if(empty($v))
			      		{
				      		continue;
			      		}
			      		
			      		$v = \deserialize($v);
			      		
			      		if(!is_array($v))
				      	{
				      		if(\Contao\Validator::isStringUuid($v))
							{
								$uuid = $v;
								$bin = null;
								if($this->arrTl_files[$uuid])
								{
									$strFile = $this->arrTl_files[$uuid];
									if(file_exists(TL_ROOT.'/'.$strFile))
									{
										$objFile = \Dbafs::addResource($strFile);
										if($objFile)
										{
											$bin = $objFile->uuid;
										}
									}
								}
								else
								{
									$bin = \Contao\StringUtil::uuidToBin($uuid);
								}
								
								if($bin !== null)
								{
									$arrWizard['values'][$field] = \Contao\StringUtil::binToUuid($bin);
								}
							}
						}
						// binary in multiSRC fields
				   		else if(in_array($field, $GLOBALS['PCT_CUSTOMELEMENTS']['multiSRC_fields']) && is_array($v))
				   		{
				   			$arrValue = \deserialize($v);
				   			if(is_array($arrValue) && count($arrValue) > 0)
				   			{
				   				$tmp = array();
				   				foreach($arrValue as $v)
				   				{
				   					$bin = null;
									if($this->arrTl_files[$uuid])
									{
										$strFile = $this->arrTl_files[$uuid];
										if(file_exists(TL_ROOT.'/'.$strFile))
										{
											$objFile = \Dbafs::addResource($strFile);
											if($objFile)
											{
												$bin = $objFile->uuid;
											}
										}
									}
									else
									{
										$bin = \Contao\StringUtil::uuidToBin($uuid);
									}
									
									if($bin !== null)
									{
										$tmp[] = \Contao\StringUtil::binToUuid($bin);
									}
				   				}
				   				
				   				$arrWizard['values'][$field] = serialize($tmp);
				   				unset($tmp);
				   			}
				   		}
					}
					
					$row['data_blob'] = serialize($arrWizard);
				}
				
				// INSERT
				$objStatement = $objDatabase->prepare("INSERT INTO ".$strTable." %s")->set($row)->execute();
				$_insertId = $objStatement->__get('insertId');
				
				$arrNewRecords[$strTable][] = $_insertId;
				
				// store as new pid
				$arrPids[$strTable][$oldId] = $_insertId; 
			}
		}
		
		// log new records
		unset($arrNewRecords['tl_pct_customelement_vault']);
		if(count($arrNewRecords) > 0)
		{
			foreach($arrNewRecords as $strTable => $ids)
			{
				\Contao\System::log('New set "'.$strName.'" included in '.$strTable.' (ids:'.implode(',', $ids).')',__METHOD__,TL_GENERAL);
			}
		}
			
		return true;
	}
	
	
	/**
	 * Prepare the sql array
	 */
	protected function prepareData($strTable, $arrData=array(), array $arrReturn=array())
	{
		$objDatabase = \Contao\Database::getInstance();
		$arrRecords = $arrData[$strTable];
		
		if(!is_array($arrRecords) || empty($arrRecords) || !$objDatabase->tableExists($strTable))
		{
			return $arrReturn;
		}
		
		// load the datacontainer
		if(!$GLOBALS['TL_DCA'][$strTable]['config'])
		{
			\Contao\Controller::loadDataContainer($strTable);
		}
		
		// parent table
		$strPtable = '';
		if(strlen($GLOBALS['TL_DCA'][$strTable]['config']['ptable']) > 0)
		{
			$strPtable = $GLOBALS['TL_DCA'][$strTable]['config']['ptable'];
		}
		
		// dynamicPtable
		$blnDynamic = (boolean)$GLOBALS['TL_DCA'][$strTable]['config']['dynamicPtable'];
		if($strTable == 'tl_pct_customelement_vault')
		{
			$blnDynamic = true;
		}
		
		foreach($arrRecords as $i => $data)
		{
			$intPid = 0;
			
			$fields = $data['fields'];
			$values = $data['values'];
			

			
			// build set array
			$row = array_combine( array_values($fields),array_values($values));
			
			// update timestamp
			if(isset($row['tstamp']))
			{
				$row['tstamp'] = $this->intTime;
			}
			
			// has a ptable field
			if($blnDynamic && strlen($row['ptable']) > 0)
			{
				$strPtable = $row['ptable'];
			}
			// has a source field
			else if($blnDynamic && strlen($row['source']) > 0)
			{
				$strPtable = $row['source'];
			}
			
			// append an info to the title
			if(strlen($row['title']) > 0 && strlen($GLOBALS['PCT_THEMER']['appendTitle']) > 0)
			{
				$row['title'] .= sprintf( $GLOBALS['PCT_THEMER']['appendTitle'],$this->getName(),\Contao\System::parseDate('d.m.Y H:i',time()) );
			}
			else if(strlen($row['name']) > 0 && strlen($GLOBALS['PCT_THEMER']['appendTitle']) > 0)
			{
				$row['name'] .= sprintf( $GLOBALS['PCT_THEMER']['appendTitle'],$this->getName(),\Contao\System::parseDate('d.m.Y H:i',time()) );
			}
			
			// remember the old id
			$oldId = $row['id'];
			unset($row['id']);
			
			// find pid
			if(strlen($strPtable) > 0)
			{
				if($row['pid'] > 0 && $this->arrPids[$strPtable][$row['pid']] > 0)
				{
					$intPid = $this->arrPids[$strPtable][$row['pid']];
				}
				else if($strTable == 'tl_module')
				{
					$intPid = $this->intTheme;
				}
			}
			// nested record of same table in MODE 5
			else if(in_array($GLOBALS['TL_DCA'][$strTable]['list']['sorting']['mode'],array(5)) && $row['pid'] > 0 && $this->arrPids[$strTable][$row['pid']] > 0)
			{
				$intPid = $this->arrPids[$strTable][$row['pid']];
			}
			
			// set new pid
			if(isset($row['pid']))
			{
				$row['pid'] = $intPid;
			}
			
			// relink redirects
			foreach($GLOBALS['PCT_CUSTOMELEMENTS']['singleJumpTo_fields'] as $field)
			{
				$oldJumpTo = $row[$field];
				if($oldJumpTo > 0 && $this->arrPids['tl_page'][$oldJumpTo] > 0 && $oldJumpTo !=  $this->arrPids['tl_page'][$oldJumpTo])
				{
					$row[$field] = $this->arrPids['tl_page'][$oldJumpTo];
				}
			}
			
			// relink multiple page selections
			foreach($GLOBALS['PCT_CUSTOMELEMENTS']['multiJumpTo_fields'] as $field)
			{
				$pages = \deserialize($row[$field]);
				if(!empty($pages)|| !is_array($pages))
				{
					continue;
				}
				
				$tmp = array();
				foreach($pages as $oldPageId)
				{
					$tmp[] = $this->arrPids['tl_page'][$oldPageId];
				}
				$row[$field] = $tmp;
				unset($tmp);
			}
			
			// INSERT
			$objStatement = $objDatabase->prepare("INSERT INTO ".$strTable." %s")->set($row)->execute();
			$_insertId = $objStatement->__get('insertId');
			
			$arrReturn[$strTable][] = $_insertId;
			
			$this->arrNewRecords[$strTable][] = $_insertId;
			
			// store as new pid
			$this->arrPids[$strTable][$oldId] = $_insertId;
		}
			
		return $arrReturn;
	}
	
	
	protected function getNextHighestSorting($strTable,$intSorting,$strPtable='')
	{
		$objResult = \Contao\Database::getInstance()
		->prepare("SELECT * FROM ".$strTable." WHERE pid=? AND sorting=?".(strlen($strPtable) > 0 ? " AND ptable=?" : "") )
		->limit(1)->execute($intSorting,$strPtable);
		if($objResult->numRows < 1)
		{
			return $intSorting;
		}
		else
		{
			return $this->getNextHighestSorting($strTable,$intSorting + 64,$strPtable);
		}
	}
	
}