<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2013
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  revolutionslider
 * @link  http://contao.org
 * @license  http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Backend;

/**
 * Class file
 * TableContent
 * Provide methods for table tl_content in do='revolutionslider'
 */
class TableContent extends \Backend
{
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser','User');
	}

	/**
	 * Check permissions to edit table tl_content
	 */
	public function checkPermission()
	{
		if ($this->User->isAdmin)
		{
			return;
		}

		// Set the root IDs
		if (!is_array($this->User->revolutionslider_slides) || empty($this->User->revolutionslider_slides))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->revolutionslider_slides;
		}
		
		// show all sliders if user has richts to create and no specific slider is selected
		if(in_array('create', $this->User->revolutionslider_slidesp) && empty($this->User->revolutionslider_slides))
		{
			return;
		}
		
		$objInput = \Input::getInstance();
		
		// Check the current action
		switch (\Input::get('act'))
		{
			case '':
			case 'paste':
			case 'create':
			case 'select':
			case 'show':
				// Allow
				break;
	
			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
			case 'cutAll':
			case 'copyAll':
				// Check access to the parent element if a content element is moved
				if (($objInput->get('act') == 'cutAll' || $objInput->get('act') == 'copyAll') && !$this->checkAccessToElement(Input::get('pid'), $root, (Input::get('mode') == 2)))
				{
					$this->redirect('contao/main.php?act=error');
				}
	
				$objCes = \Database::getInstance()->prepare("SELECT id FROM tl_content WHERE ptable='tl_revolutionslider_slides' AND pid=?")
				->execute(CURRENT_ID);
	
				$session = \Session::getInstance()->getData();
				$session['CURRENT']['IDS'] = array_intersect($session['CURRENT']['IDS'], $objCes->fetchEach('id'));
				\Session::getInstance()->setData($session);
				break;
	
			case 'cut':
			case 'copy':
				// Check access to the parent element if a content element is moved
				if (!$this->checkAccessToElement($objInput->get('pid'), $root, ($objInput->get('mode') == 2)))
				{
					$this->redirect('contao/main.php?act=error');
				}
				// NO BREAK STATEMENT HERE
	
			default:
				// Check access to the content element
				if (!$this->checkAccessToElement(\Input::get('id'), $root))
				{
					$this->redirect('contao/main.php?act=error');
				}
				break;
		}
	}


	/**
	 * Check access to a particular content element
	 * @param integer
	 * @param array
	 * @param boolean
	 * @return boolean
	 */
	protected function checkAccessToElement($id, $root, $blnIsPid=false)
	{
		if ($blnIsPid)
		{
			$objArchive = \Database::getInstance()->prepare("SELECT a.id, n.id AS nid FROM tl_revolutionslider_slides n, tl_revolutionslider a WHERE n.id=? AND n.pid=a.id")
			->limit(1)
			->execute($id);
		}
		else
		{
			$objArchive = \Database::getInstance()->prepare("SELECT a.id, n.id AS nid FROM tl_content c, tl_revolutionslider_slides n, tl_revolutionslider a WHERE c.id=? AND c.pid=n.id AND n.pid=a.id")
			->limit(1)
			->execute($id);
		}
		
		// Invalid ID
		if ($objArchive->numRows < 1)
		{
			$this->log('Invalid slides content element ID ' . $id, 'tl_revolutionslider_slides checkAccessToElement()', TL_ERROR);
			return false;
		}
		
		// The slide is not mounted
		if (!in_array($objArchive->nid, $root))
		{
			$this->log('Not enough permissions to modify slide ID ' . $objArchive->nid . ' in slider ID ' . $objArchive->id, 'tl_revolutionslider_slides checkAccessToElement()', TL_ERROR);
			return false;
		}

		return true;
	}


	/**
	 * Return all slider ids as array
	 * @param object
	 * @return array
	 */
	public function getSliders(\DataContainer $objDC)
	{
		$objSliders = \RevolutionSlider\Models\Slider::findAll();
		
		if($objSliders === null)
		{
			return array();
		}
		
		$arrReturn = array();
		while($objSliders->next())
		{
			$arrReturn[$objSliders->id] = $objSliders->title . ' (id:'.$objSliders->id.')';
		}
		
		asort($arrReturn);

		return $arrReturn;
	}


	/**
	 * Get all slides of the current slider an return as array
	 * @param object
	 * @return array
	 */
	public function getSlides(\DataContainer $objDC)
	{
		$objDatabase = \Database::getInstance();

		$objSlides = $objDatabase->prepare(
			"SELECT * FROM tl_revolutionslider_slides WHERE pid=
			(
				SELECT pid FROM tl_revolutionslider_slides WHERE id=?
			) AND id!=? ORDER BY sorting"
		)->execute($objDC->activeRecord->pid,$objDC->activeRecord->pid);

		if($objSlides->numRows < 1)
		{
			return array();
		}

		$arrReturn = array();
		while($objSlides->next())
		{
			$arrReturn[$objSlides->id] = $objSlides->title . ' (id:' .$objSlides->id.')';
		}

		return $arrReturn;
	}

	/**
	 * Modify the palettes depending on selected video type
	 * @param object
	 * @return object
	 */
	public function modifyPalettes(\DataContainer $objDC)
	{
		if(\Input::getInstance()->get('act') != 'edit')
		{
			return $objDC;
		}

		$objDatabase = \Database::getInstance();
		$objActiveRecord = $objDC->activeRecord;
		if(!$objActiveRecord)
		{
			$objActiveRecord = $objDatabase->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		}
		
		if($objActiveRecord->type == 'revolutionslider_video')
		{
			switch($objActiveRecord->revolutionslider_videoType)
			{
				case 'local':
					$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video'] = str_replace(array('url'),'',$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video']);
					break;
				case 'youtube':
				case 'vimeo':
				case 'external':
					$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video'] = str_replace(array('multiSRC'),'',$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video']);
					
					// remove the singleSRC field
					unset($GLOBALS['TL_DCA']['tl_content']['fields']['singleSRC']);
					break;
				default:

					break;
			}

			#fix: overwrite default contao fields only when revolutionslider_video is active
			// singleSRC
			$GLOBALS['TL_DCA']['tl_content']['fields']['singleSRC']['label'] = &$GLOBALS['TL_LANG']['tl_content']['revolutionslider_video_singleSRC'];
			$GLOBALS['TL_DCA']['tl_content']['fields']['singleSRC']['eval']['mandatory'] = false;
			$GLOBALS['TL_DCA']['tl_content']['fields']['multiSRC']['eval']['mandatory'] = false;

			// url
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['label'] = &$GLOBALS['TL_LANG']['tl_content']['revolutionslider']['url'];
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['inputType'] = 'text';
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['wizard'] = null;
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['eval'] = array('mandatory'=>false);
		}
		
		// RS Video standalone
		if($objActiveRecord->type == 'revolutionslider_video' && $objActiveRecord->ptable == 'tl_article')
		{
			$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video']  = '{type_legend},type,headline;{source_legend},revolutionslider_videoType,multiSRC,url,singleSRC;{video_settings_legend},revolutionslider_data_autoplay,revolutionslider_video_loop,revolutionslider_video_controls,revolutionslider_video_size;{template_legend:hide},customTpl;{protected_legend:hide},protected;{expert_legend:hide},guests,invisible,cssID,space';

		}
		
		if($objActiveRecord->type == 'revolutionslider_text')
		{
			$GLOBALS['TL_DCA']['tl_content']['fields']['text']['label'] = $GLOBALS['TL_LANG']['tl_content']['revolutionslider_text'];
			$GLOBALS['TL_DCA']['tl_content']['fields']['text']['eval'] = array('style'=>'min-height:40px;');
		}
			
		if(!in_array($objActiveRecord->type, array('revolutionslider_text','revolutionslider_hyperlink','revolutionslider_image') ) && !$GLOBALS['REVOLUTIONSLIDER']['alwaysShowPos9Grid'])
		{
			unset($GLOBALS['TL_DCA']['tl_content']['fields']['revolutionslider_data_pos9grid']);
		}
		else
		{
			$GLOBALS['TL_DCA']['tl_content']['fields']['revolutionslider_data_position']['label'] = $GLOBALS['TL_LANG']['tl_content']['revolutionslider_data_position_offset'];
		}
		
		return $objDC;
	}
	
	
	/**
	 * Hide revolution slider fields outside its backend module
	 * @param object
	 */
	public function hideFields(\DataContainer $objDC)
	{
		// fix #15: hide slider fields in editAll and overrideAll mode
		if(\Input::get('do') != 'revolutionslider' && in_array(\Input::get('act'),array('editAll','overrideAll')) )
		{
			foreach($GLOBALS['TL_DCA']['tl_content']['fields'] as $k => $v)
			{
				if(strlen(strpos($k, 'revolutionslider')) < 1 || $k === 'revolutionslider')
				{
					continue;
				}
				unset($GLOBALS['TL_DCA']['tl_content']['fields'][$k]);
			}
		}
	}

}
