<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 * @license		http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Core;

/**
 * Class files
 * RevolutionSliderSlide
 */
class Slide extends \Controller
{
	/**
	 * Content elements array
	 * @var array
	 */
	protected $arrContent = array();
	
	/**
	 * Attributes array
	 * @var array
	 */
	protected $arrAttributes = array();
	
	/**
	 * Data Array
	 * @var array
	 */
	protected $arrData = array();

	/**
	 * Create new instance
	 * @param array
	 */ 
	public function __construct($arrData)
	{
		$this->arrData = $arrData;
		$this->arrContent = $this->prepareContent($arrData['id']);
		$this->arrAttributes = array
		(
			'data-transition'	=> $arrData['transition'],
			'data-slotamount'	=> $arrData['slotamount'],
		);
		
		// fallback transition
		if(!$this->arrAttributes['data-transition'])
		{
			$objSlider = $this->getSlider();
			$this->arrAttributes['data-transition'] = $objSlider->get('transition');
		}
		
		// delay
		if($arrData['delay'] > 0)
		{
			$this->arrAttributes['data-delay'] = $arrData['delay'] * 1000;
		}
		
		// masterspeed
		if($arrData['masterspeed'] > 0)
		{
			$this->arrAttributes['data-masterspeed'] = $arrData['masterspeed'] * 1000;
		}
		
		// link slide
		if(strlen($arrData['slideUrl']) > 0)
		{
			$this->arrAttributes['data-link'] = $arrData['slideUrl'];
			$this->arrAttributes['data-target'] = ($arrData['newWindow'] ? '_blank' : '');
		}
		
		// generate thumbs
		$objSlider = $this->getSlider();
		if($objSlider->get('addThumbs'))
		{
			$this->arrAttributes['data-thumb'] = $this->getThumb();
		}
		
		// slide title
		$this->arrAttributes['data-title'] = $arrData['title'];
	}
	
	/**
	 * Set Data
	 * @param array
	 */
	public function setData($arrData)
	{
		$this->arrData = $arrData;
	}

	/**
	 * Getters
	 * @param string
	 */
	public function get($strKey)
	{
		if(isset($this->$strKey))
		{
			return $this->$strKey;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			return $this->arrData[$strKey];
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Setters
	 * @param string
	 * @param mixed
	 */
	public function set($strKey, $varValue)
	{
		if(isset($this->$strKey))
		{
			$this->$strKey = $varValue;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			$this->arrData[$strKey] = $varValue;
		}
		else
		{
			return null;
		}
	}
		
	/**
	 * Prepare the slide
	 * Fetch all content elements
	 * @param array
	 * @return array
	 */
	public function prepareContent($intSlide)
	{
		// get raw content data 
		$arrRawContent = $this->findContentBySlide($intSlide);
		
		if(empty($arrRawContent))
		{
			return array();
		} 
		
		// generate content
		$arrContent = array();
		foreach($arrRawContent as $cte)
		{
			// attributes
			$data_position = deserialize($cte['revolutionslider_data_position']);
			$data_position9 = explode(' ',$cte['revolutionslider_data_pos9grid']);
			$arrAttributes = array
			(
				'data-easing'	=> $cte['revolutionslider_data_easing'],
				'data-speed'	=> ($cte['revolutionslider_data_speed'] > 0 ? $cte['revolutionslider_data_speed'] * 1000 : 0), // sec to ms
				'data-x'		=> (strlen($data_position[0]) ? $data_position[0]: $data_position9[0]),
				'data-y'		=> (strlen($data_position[1]) ? $data_position[1]: $data_position9[1]),
				'data-start'	=> ($cte['revolutionslider_data_start'] > 0 ? $cte['revolutionslider_data_start'] * 1000 : 100),
				'data-elementdelay' => ($cte['revolutionslider_data_delay'] > 0 ? $cte['revolutionslider_data_delay'] / 100 : 0),
			);
			
			if($cte['revolutionslider_data_split'] && $cte['revolutionslider_data_delay'] > 0)
			{
				$arrAttributes['data-elementdelay'] = $cte['revolutionslider_data_delay'];
			}
			
			// h offset
			if(isset($data_position[0]) && strlen($data_position9[0]) > 0)
			{
				$arrAttributes['data-x'] = $data_position9[0];
				$arrAttributes['data-hoffset'] = $data_position[0];
			}
			
			// v offset
			if(isset($data_position[1]) && strlen($data_position9[1]) > 0)
			{
				$arrAttributes['data-y'] = $data_position9[1];
				$arrAttributes['data-voffset'] = $data_position[1];
			}
			
			// Custom Incoming animation
			if($cte['revolutionslider_data_animation_start'] == 'customin')
			{
				$data_rotation = deserialize($cte['revolutionslider_data_rotation']);
				$data_scale = deserialize($cte['revolutionslider_data_scale']);
				$data_skew = deserialize($cte['revolutionslider_data_skew']);
				$arrCustomin = array
				(
					'x' 				=> (strlen($data_position[0]) ?  $data_position[0]: '0'),
					'y'					=> (strlen($data_position[1]) ?  $data_position[1]: '0'),
					'z'					=> (strlen($data_position[2]) ?  $data_position[2]: '0'),
					'rotationX'			=> (strlen($data_rotation[0]) ?  $data_rotation[0]: '0'),
					'rotationY'			=> (strlen($data_rotation[1]) ?  $data_rotation[1]: '0'),
					'rotationZ'			=> (strlen($data_rotation[2]) ?  $data_rotation[2]: '0'),
					'scaleX'			=> (strlen($data_scale[0]) ?  $data_scale[0]: '1'),
					'scaleY'			=> (strlen($data_scale[1]) ?  $data_scale[1]: '1'),
					'skewX'				=> (strlen($data_skew[0]) ?  $data_skew[0]: '0'),
					'skewY'				=> (strlen($data_skew[1]) ?  $data_skew[1]: '0'),
					'opacity'			=> ($cte['revolutionslider_data_opacity'] > 0 ? $cte['revolutionslider_data_opacity'] : '0'),
				);
				
				if($cte['revolutionslider_data_perspective'] > 0)
				{
					$arrCustomin['transformPerspective'] = $cte['revolutionslider_data_perspective'];
				}
				
				$data_origin = deserialize($cte['revolutionslider_data_origin']);
				if(strlen($data_origin[0]) || strlen($data_origin[1]))
				{
					$arrCustomin['transformOrigin'] = ($data_origin[0] > 0 ? $data_origin[0].'%': '0%').' '.($data_origin[1] > 0 ? $data_origin[1].'%': '0%');
				}
				
				$strCustomin = '';
				foreach($arrCustomin as $k => $v)
				{
					$strCustomin .= $k.':'.$v.';';
				}
				$strCustomin = substr($strCustomin,0,-1);
				$arrAttributes['data-customin'] = $strCustomin;
			}
			
			// typo effects
			if($cte['revolutionslider_data_split'] && strlen($arrAttributes['data-customin']) > 0)
			{
				$arrAttributes['data-splitin'] = $cte['revolutionslider_data_split'];
				$arrAttributes['data-frames'] = (strlen($cte['revolutionslider_data_frames']) ?  $cte['revolutionslider_data_frames']: '');
			}
			
			// out animation attributes
			if($cte['revolutionslider_OUT'] > 0)
			{
				// use endelementdealy for text effects
				if($cte['revolutionslider_data_start_OUT'] > 0)
				{
					if($cte['revolutionslider_data_split'])
					{
						$arrAttributes['data-endelementdelay'] = $cte['revolutionslider_data_start_OUT'];
					}
					else
					{
						$arrAttributes['data-end'] = $cte['revolutionslider_data_start_OUT'] * 1000;
					}
				}
				$arrAttributes['data-endspeed'] 		= ($cte['revolutionslider_data_speed_OUT'] > 0 ? $cte['revolutionslider_data_speed_OUT'] * 1000 : 0);
				$arrAttributes['data-endeasing'] 		= (strlen($cte['revolutionslider_data_easing_OUT']) ? $cte['revolutionslider_data_easing_OUT'] : '');
				
				if($cte['revolutionslider_data_animation_end'] == 'customout')
				{
					// Custom Outgoing animation
					$data_position = deserialize($cte['revolutionslider_data_position_OUT']);
					$data_rotation = deserialize($cte['revolutionslider_data_rotation_OUT']);
					$data_scale = deserialize($cte['revolutionslider_data_scale_OUT']);
					$data_skew = deserialize($cte['revolutionslider_data_skew_OUT']);
				
					$arrCustomout = array
					(
						'x' 				=> (strlen($data_position[0]) ?  $data_position[0]: '0'),
						'y'					=> (strlen($data_position[1]) ?  $data_position[1]: '0'),
						'z'					=> (strlen($data_position[2]) ?  $data_position[2]: '0'),
						'rotationX'			=> (strlen($data_rotation[0]) ?  $data_rotation[0]: '0'),
						'rotationY'			=> (strlen($data_rotation[1]) ?  $data_rotation[1]: '0'),
						'rotationZ'			=> (strlen($data_rotation[2]) ?  $data_rotation[2]: '0'),
						'scaleX'			=> (strlen($data_scale[0]) ?  $data_scale[0]: '1'),
						'scaleY'			=> (strlen($data_scale[1]) ?  $data_scale[1]: '1'),
						'skewX'				=> (strlen($data_skew[0]) ?  $data_skew[0]: '0'),
						'skewY'				=> (strlen($data_skew[1]) ?  $data_skew[1]: '0'),
						'opacity'			=> ($cte['revolutionslider_data_opacity'] > 0 ? $cte['revolutionslider_data_opacity'] : '0'),
					);
					$strCustomout = '';
					foreach($arrCustomout as $k => $v)
					{
						$strCustomout .= $k.':'.$v.';';
					}
					$strCustomout = substr($strCustomout,0,-1);
					$arrAttributes['data-customout'] = $strCustomout;
					
					if($cte['revolutionslider_data_split_OUT'])
					{
						$arrAttributes['data-splitout'] = $cte['revolutionslider_data_split_OUT'];
					}
				}
			}
			
			// link element to slide
			if($cte['revolutionslider_data_linktoslide'] > 0)
			{
				$objSlide = \RevolutionSlider\Models\Slides::findByPk($intSlide);
				if($objSlide !== null)
				{
					$objSlides = \Database::getInstance()->prepare("SELECT * FROM tl_revolutionslider_slides WHERE pid=? ORDER BY sorting")->execute($objSlide->pid);
					
					$i = 1;
					while($objSlides->next())
					{
						if($cte['revolutionslider_data_linktoslide'] == $objSlides->id)
						{
							$arrAttributes['data-linktoslide'] = $i;
						}
						$i++;
					}
				}
			}
			
			// generate the content element
			$strCte = $this->replaceInsertTags('{{insert_content::'.$cte['id'].'}}');
			
			// strip the TEMPLATE START comments from the html string
			if(strlen(strpos($strCte,'TEMPLATE START')) > 0 && \Config::get('debugMode'))
			{
				$strCte = preg_replace('<!--(.*?)-->','', $strCte);
				$strCte = str_replace('<>','', $strCte);
			}
			
			// inject attributes in content element
			$strAttributes = '';
			foreach($arrAttributes as $attr => $value)
			{
				$strAttributes .= $attr .'="'.$value.'" ';
			}
			trim($strAttributes);
			
			// since we only work on html elements we can easily just replace everything in the first <...>
			$preg = preg_match('/<(.*?)\>/', $strCte,$result);
			if($preg)
			{
				$search = $result[0];
				$replace = sprintf('<%s %s>',$result[1], $strAttributes);
				$strCte = str_replace($search, $replace, $strCte);
			}
			
			// add must have classes
			$preg = preg_match('/class="(.*?)\"/', $strCte,$result);
			if($preg)
			{
				$arrClass = explode(' ', trim($result[1]));
				
				$arrClass[] = 'tp-caption';
				#$arrClass[] = 'tp-resizeme';
				#$arrClass[] = 'rs-parallaxlevel-0';
				
				if(strlen($arrAttributes['data-customin']) > 0)
				{
					$arrClass[] = 'customin';
				}
				
				if($cte['revolutionslider_OUT'] && strlen($arrAttributes['data-customout']) > 0)
				{
					$arrClass[] = 'customout';
				}
				
				// add start animation class
				if(!in_array($cte['revolutionslider_data_animation_start'], $arrClass))
				{
					$arrClass[] = $cte['revolutionslider_data_animation_start'];
				}
				
				// add end animation class
				if(!in_array($cte['revolutionslider_data_animation_end'], $arrClass))
				{
					$arrClass[] = $cte['revolutionslider_data_animation_end'];
				}
				
				$search = $result[0];
				$strCte = str_replace($search, 'class="'.implode(' ', array_unique($arrClass)).'"', $strCte);
			}
			
			$arrContent[] = $strCte;
		}
		
		return $arrContent;
	}
	 
	/**
	 * Find content elements of a slide
	 * @param integer
	 * @return array
	 */
	protected function findContentBySlide($intSlide)
	{
		$objCte =  \Database::getInstance()->prepare("SELECT * FROM tl_content WHERE ptable='tl_revolutionslider_slides' AND pid=? AND invisible!=1 ORDER BY sorting")->execute($intSlide);
			
		return $objCte->fetchAllAssoc();
	}


	/**
	 * Generate the current thumbnail from the main image or from the slide image and return the filepath
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string	path to thumb
	 */
	protected function getThumb()
	{
		if($this->get('singleSRC') == '' && $this->get('singleSRC_thumb') == '')
		{
			return '';
		}
		
		$objSlider = $this->getSlider();
		
		$src = $this->get('singleSRC_thumb');
		if($src == '')
		{
			$src = $this->get('singleSRC');
		}
		
		$size = deserialize($objSlider->get('thumbSize'));
		
		$file = \FilesModel::findByUuid($src)->path;
		
		return $this->getImage($file,$size[0],$size[1],$size[2]);
	}


	/**
	 * Return the parent slider object
	 * Optional allow to generate the slides
	 * @return object	RevolutionSlider
	 */
	public function getSlider()
	{
		return \RevolutionSlider\Core\Factory::findById($this->get('pid'),false);
	}
	
}