<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright 	Tim Gatzky 2015, Premium Contao Webworks, Premium Contao Themes
 * @author  	Tim Gatzky <info@tim-gatzky.de>
 * @package  	pct_revolutionslider
 * @link 		http://contao.org
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Frontend;


/**
 * Class file
 */
class ContentRevolutionSliderText extends \ContentElement
{
	/**
	 * Template
	 * @var
	 */
	protected $strTemplate = 'ce_revolutionslider_text';

	/**
	 * Display wildcard
	 */
	public function generate()
	{
		if (TL_MODE == 'BE')
		{
			$this->Template = new \BackendTemplate('be_wildcard');
			$this->Template->wildcard = '### REVOLUTION SLIDER TEXT ###'.($this->text ? '<p>'.$this->text.'</p>' : '');
			$this->Template->title = $this->headline;
			
			return $this->Template->parse();
		}
		
		if(strlen($this->customTpl) > 0 && $this->customTpl != $this->strTemplate)
		{
			$this->strTemplate = $this->customTpl;
		}

		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		$arrCssID = $this->cssID;
		$arrClass = explode(' ', $arrCssID[1]);
		$arrClass[] = $this->strTemplate;
		$arrClass[] = 'caption';
		$arrClass[] = 'tp-caption';
		if($this->revolutionslider_text_bold)
		{
			$arrClass[] = 'bold';
		}
		if($this->revolutionslider_text_invertcolor)
		{
			$arrClass[] = 'invertcolor';
		}
		if($this->revolutionslider_text_italic)
		{
			$arrClass[] = 'italic';
		}
		
		$arrCssID[1] = trim(implode(' ', array_unique($arrClass)));
		$this->cssID = $arrCssID;
		
		$arrStyles = array
		(
			'font-size'	=> (isset($this->revolutionslider_text_fontsize) ? $this->revolutionslider_text_fontsize.'px' : '14px'), 
		);
		
		$this->Template->styles = $this->prepareStyles($arrStyles);
	}
	
	
	/**
	 * Generate the inline style string
	 * @param array
	 * @return string
	 */
	protected function prepareStyles($arrStyles)
	{
		if(empty($arrStyles))
		{
			return '';
		}
		
		$strReturn = '';
		foreach($arrStyles as $k => $v)
		{
			$strReturn .= $k.':'.$v.';';
		}
		$strReturn = substr($strReturn,0,-1);
		
		return $strReturn;
	}
}