<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  revolutionslider
 * @link  http://contao.org
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Frontend;

/**
 * Class file
 */
class ContentRevolutionSliderVideo extends \ContentElement
{
	/**
	 * Template
	 * @var
	 */
	protected $strTemplate = 'ce_revolutionslider_video';

	/**
	 * Display wildcard
	 */
	public function generate()
	{
		if($this->ptable == 'tl_article')
		{
			$this->isStandalone = true;
		}
		
		if (TL_MODE == 'BE')
		{
			// display video sources
			$strInfo = '';
			if($this->revolutionslider_videoType == 'external')
			{
				$strInfo = '<p class="tl_gray">'.$this->url.'</p>';
			}
			else if($this->revolutionslider_videoType == 'local' && strlen($this->multiSRC) > 0)
			{
				$arrSources = array();
				$arrFiles = deserialize($this->multiSRC);
				$objFiles = \FilesModel::findMultipleByIds($arrFiles);
				if($objFiles)
				{
					$arrSources = $objFiles->fetchEach('path');
	
					$strInfo .= '<p class="tl_gray">';
					foreach($arrSources as $src)
					{
						$strInfo .= '<div class="tl_gray no-padding">'.$src.'</div>';
					}
					$strInfo.='</p>';
				}
			}

			$this->Template = new \BackendTemplate('be_wildcard');
			$this->Template->wildcard = '### REVOLUTION SLIDER VIDEO ###'. $strInfo;;
			$this->Template->title = $this->headline;

			return parent::generate();
		}
				
		if(strlen($this->customTpl) > 0 && $this->customTpl != $this->strTemplate)
		{
			$this->strTemplate = $this->customTpl;
		}
		
		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		$arrCssID = deserialize($this->cssID);
		$arrClass = explode(' ', $arrCssID[1]);
		
		$this->Template = new \FrontendTemplate($this->strTemplate);
		$this->Template->setData($this->arrData);
		
		$arrSize = deserialize($this->revolutionslider_video_size) ?: array();
		
		$this->Template->startWidth = $arrSize[0] ?: "0";
		$this->Template->startHeight = $arrSize[1] ?: "0";
		$this->Template->selector = 'revolutionslidervideo'.$this->id;
		
		$arrAttributes = array();
		$arrSources = array();
		if($this->revolutionslider_videoType == 'external' && strlen($this->url) > 0)
		{
			$src = $this->url;
			
			$w = ($arrSize[0] > 0 ? $arrSize[0] : '100%');
			$h = ($arrSize[1] > 0 ? $arrSize[1] : '100%');
			
			$tmp = parse_url($src);
			$preg = preg_match("/&(.*?)\&/", str_replace('.','&',$tmp['host']) ,$match);
			$host = $match[1];
			switch($host)
			{
				case 'youtube':
				case 'youtu': // youtu.be
					// make youtube video player accessible with js
					$src = $this->addToAnyUrl($src,'enablejsapi=1');
					break;
				case 'vimeo':
					// make vimeo video player accessible with js
					$src = $this->addToAnyUrl($src,'api=1');
					break;
			}
			
			$iframe_params = array();
			if($this->revolutionslider_video_fullscreen || $this->isStandalone)
			{
				$iframe_params[] = 'allowfullscreen';
			}
			
			$this->Template->iframe_params = $iframe_params;
			$this->Template->iframe =  '<iframe src="'.$src.'" width="'.$w.'" height="'.$h.'" '.implode(' ', $iframe_params).'></iframe>';
		}
		// local HTML5 video files
		else if($this->revolutionslider_videoType == 'local' && strlen($this->multiSRC) > 0)
		{
			if($strDataSetup)
			{
				$arrAttributes[] = 'data-setup="{'.$strDataSetup.'}"';
			}
			$arrFiles = deserialize($this->multiSRC);
			$objFiles = \FilesModel::findMultipleByIds($arrFiles);
			if(!$objFiles)
			{
				return '';
			}

			$files = $objFiles->fetchEach('path');
			
			foreach($files as $source)
			{
				$ext = pathinfo($source, PATHINFO_EXTENSION);
					
				if(in_array($ext, array('flv')) )
				{
					continue;
				}
				
				if($ext == 'ogv') {$ext = 'ogg';}
				
				$arrSources[] = array
				(
					'src' 	=> $source,
					'mime'	=> 'video/'.$ext,
				);
			}
		}
		else
		{
			// HOOK here?
		}
		
		// poster
		if($this->singleSRC)
		{
			$strPoster = \FilesModel::findByPk($this->singleSRC)->path;
			$this->Template->poster = $strPoster;
		}
		
		// fix #8 autoplay
		if($this->revolutionslider_data_autoplay)
		{
			$arrAttributes[] = 'data-autoplay="true"';
		}
		
		// next slide at the end
		if($this->revolutionslider_data_nextslideatend)
		{
			$arrAttributes[] = 'data-nextslideatend="true"';
		}
		
		// stretch video
		$arrClass[] = (!$this->revolutionslider_video_boxed ? 'fullscreenvideo' : 'boxed');
		$arrClass[] = 'caption';
		$arrClass[] = 'tp-caption';
		
		if($this->revolutionslider_video_fullscreen)
		{
			$this->Template->fullscreen = true;
			#$arrClass[] = 'fullcoveredvideo';
			$arrAttributes[] = 'data-forcecover="1"';
		}
		
		if($this->isStandalone && $arrSize[0] < 1)
		{
			$arrClass[] = "fullscreenvideo";
		}
		
		$arrCssID[1] = implode(' ', array_filter(array_unique($arrClass),'strlen') );
		
		$this->cssID = $arrCssID;
		$this->Template->class = $arrCssID[1];
		$this->Template->attributes = trim(implode(' ', $arrAttributes));
		$this->Template->files = $arrSources;
		$this->Template->source = $this->revolutionslider_videoType;
		$this->Template->controls = ($this->revolutionslider_video_controls > 0 ? 'controls' : '');
		$this->Template->loop = ($this->revolutionslider_video_loop > 0 ? 'loop' : '');
		$this->Template->no_video_support = $GLOBALS['TL_LANG']['REVOLUTIONSLIDER']['browser_does_not_support_video'] ? $GLOBALS['TL_LANG']['REVOLUTIONSLIDER']['browser_does_not_support_video'] : 'Your browser does not support the video tag.';
		$this->Template->boxed = ($this->revolutionslider_video_boxed ? true : false);
		$this->Template->width = $arrSize[0];
		$this->Template->height = $arrSize[1];
		$this->Template->autoplay = $this->revolutionslider_data_autoplay;

		if($this->revolutionslider_video_boxed)
		{
			$this->Template->styles = $arrSize[0] ? 'max-width:'.$arrSize[0].'px;' : '' . $arrSize[1] ? 'max-height:'.$arrSize[1].'px;' : '';
		}
		
		return $this->Template->parse();
	}


	/**
	 * Find files in folders recursive
	 * @param array
	 * @param boolean
	 * @return array
	 */
	protected function findFiles($strDir,$arrFiles=array(),$bolRecursive=true)
	{
		if(!is_dir($strDir))
		{
			return array();
		}
		
		$scan = scandir($strDir);
		if(empty($scan))
		{
			return array();
		}
		
		foreach($scan as $f)
		{
			if(strpos($f, '.') === 0)
			{
				continue;
			}
			
			if(is_dir($strDir.'/'.$f) && $bolRecursive)
			{
				$arrFiles = $this->findFiles($strDir.'/'.$f,$arrFiles);
				continue;
			}
			
			$arrFiles[] = $strDir.'/'.$f; 
		}
		
		return $arrFiles;	
	}
	
	
	/**
	 * Add param to url string
	 * @param string	Base url
	 * @param string	Query string
	 * @return string
	 */
	protected function addToAnyUrl($strUrl, $strQuery='')
	{
		$arrUrl = array();
		if(class_exists('\Contao\StringUtil'))
		{
			$arrUrl = explode('?',\StringUtil::decodeEntities($strUrl));
		}
		else
		{
			$arrUrl = explode('?',\String::decodeEntities($strUrl));
		}
		
		$strBase = $arrUrl[0];
		
		$arrParams = array();
		if(class_exists('\Contao\StringUtil'))
		{
			$arrParams = explode('?',\StringUtil::decodeEntities($strQuery));
		}
		else
		{
			$arrParams = explode('?',\String::decodeEntities($strQuery));
		}

		
		$arrQuery = array();
		$arrReturn = array();
		
		if($arrUrl[1])
		{
			$arrQuery = explode('&',$arrUrl[1]);
		}
		
		foreach($arrParams as $param)
		{
			$tmp = explode('=',$param);
			$pk = $tmp[0];
			$pv = $tmp[1];
			// add the parameter
			$arrReturn[$pk] = $pv;
			
			if(empty($arrQuery)) {continue;}
			
			foreach($arrQuery as $param)
			{
				$tmp = explode('=',$param);
				$qk = $tmp[0];
				$qv = $tmp[1];
				
				// overwrite existing params
				if($qk == $pk)
				{
					$qv = $pv;
				}
				$arrReturn[$qk] = $qv;
			}
		}
		
		// build url and return
		$strReturn = $strBase . '?';
		foreach($arrReturn as $k => $v)
		{
			$strReturn .= $k."=".$v."&";
		}
		$strReturn = substr($strReturn,0,-1);
		return $strReturn;
	}
	
	
}