<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (c) 2005-2019 Leo Feyer
 *
 * @copyright	Tim Gatzky 2019, Premium-Contao-Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package 	pct_seo_helper
 * @link    	https://contao.org, https://www.premium-contao-themes.com
 * @license 	http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */


/**
 * Namespace
 */
namespace PCT\SEO;


/**
 * Class file
 * ContaoCallbacks
 */
class ContaoCallbacks
{
	/**
	 * Add the 'loading' attribute to <img> elements
	 * @param object
	 */
	public function parseFrontendTemplateCallback($strBuffer, $strTemplate)
	{
		$strLoading = \Contao\Config::get('pct_seo_image_loading');

		if( empty($strLoading) === true )
		{
			return $strBuffer;
		}

		// find images
		if( preg_match_all('/(<img[^>]+>)/i', $strBuffer, $arrMatches) )
		{
			foreach( $arrMatches as $match )
			{
				if( empty($match) === true )
				{
					continue;
				}

				$image = $match[0];
				// skip images already processed or containing an loading attribute
				if( in_array($image,$GLOBALS['PCT_SEO']['processedImages'] ?: array() ) == true || strpos($image,'loading="') !== false )
				{
					continue;
				}
				// insert loading attribute
				$new_image = str_replace('<img','<img loading="'.$strLoading.'"',$image);
				// replace in html string
				$strBuffer = str_replace($image,$new_image,$strBuffer);
				// mark as being processed
				$GLOBALS['PCT_SEO']['processedImages'][] = $image;
				$GLOBALS['PCT_SEO']['processedImages'][] = $new_image;
			}
		}

		return $strBuffer;
	}


	/**
	 * Add page record information directly to the fe_page template
	 * @param object
	 * @param object
	 */
	public function generatePageCallback($objPage, $objLayout, $objPageRegular)
	{
		if( strlen($objPage->pct_structured_data) > 0 )
		{
			$objPageRegular->Template->pct_structured_data = \Contao\StringUtil::decodeEntities($objPage->pct_structured_data);
		}

		// HTTP1/2
		if( empty(\Contao\Config::get('pct_seo_protocol')) === false )
		{
			$strProtocol = \PCT\SEO::getProtocol(); // http1, http2
			
			// add a template variable
			$objPageRegular->Template->seo_protocol = $strProtocol;
			
			$version = str_replace(array('http','https'),'',$strProtocol);

			// http1.x
			if( version_compare($version,'2','<') )
			{
				// add |static
				$GLOBALS['TL_JAVASCRIPT'] = array_unique($GLOBALS['TL_JAVASCRIPT']);
				foreach( $GLOBALS['TL_JAVASCRIPT'] as $i => $v )
				{
					if( strpos($v,'|static') === false && file_exists(TL_ROOT.'/'.$v) )
					{
						$GLOBALS['TL_JAVASCRIPT'][$i] = $v.'|static';
					}
				}

				$GLOBALS['TL_CSS'] = array_unique($GLOBALS['TL_CSS']);
				foreach( $GLOBALS['TL_CSS'] as $i => $v )
				{
					if( strpos($v,'|static') === false && file_exists(TL_ROOT.'/'.$v) )
					{
						$GLOBALS['TL_CSS'][$i] = $v.'|static';
					}
				}
			}
			// http2.x
			else if( version_compare($version,'2','>=') )
			{
				// strip |static
				$GLOBALS['TL_JAVASCRIPT'] = array_unique($GLOBALS['TL_JAVASCRIPT']);
				foreach( $GLOBALS['TL_JAVASCRIPT'] as $i => $v )
				{
					$GLOBALS['TL_JAVASCRIPT'][$i] = str_replace('|static','',$v);
				}
				$GLOBALS['TL_CSS'] = array_unique($GLOBALS['TL_CSS']);
				foreach( $GLOBALS['TL_CSS'] as $i => $v )
				{
					$GLOBALS['TL_CSS'][$i] = str_replace('|static','',$v);
				}
			}
		}
	}
}