<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2019
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_theme_settings
 * @link		http://contao.org
 */
 
 
/**
 * Namespace
 */
namespace PCT\ThemeSettings;

/**
 * Imports
 */
use Contao\Validator;
use Contao\FilesModel;


/**
 * Class file
 * Core
 */
class Core
{
	/**
	 * Add the theme setting related information to the article template
	 * @param object	The template object
	 * @param object	The database result or model oobject
	 * @return object	The template object
	 */
	public static function addToTemplate($objTemplate,$objRow)
	{
		// @var object
		$objTheme = new \StdClass;
		$objTheme->raw = $objRow;
		$objTheme->layout_css = $objRow->layout_css;
		// paddings
		$objTheme->padding_top = $objRow->padding_t;
		$objTheme->padding_bottom = $objRow->padding_b;
		// margins
		$objTheme->margin_top = $objRow->margin_t;
		$objTheme->margin_bottom = $objRow->margin_b;
		
		$objTheme->ol_position = $objRow->ol_position;
		$objTheme->ol_width = $objRow->ol_width;
		$objTheme->ol_opacity = $objRow->ol_opacity;
		$objTheme->ol_bgcolor_css = $objRow->ol_bgcolor_css;
		$objTheme->color_css = $objRow->color_css;

		$arrClasses = array($objRow->layout_css,$objRow->padding_t,$objRow->padding_b,$objRow->ol_position,$objRow->ol_width,$objRow->ol_opacity,$objRow->ol_bgcolor_css,$objRow->color_css);
		
		// add parallax class
		if($objRow->background && $objRow->bgposition == 'parallax')
		{
			$objTheme->parallax = true;
			$arrClasses[] = 'parallax';
		}
		
		// collect styles
		$arrStyles = array();
		
		// background settings
		if($objRow->background)
		{
			$objTheme->background = true;
			
			if($objRow->bgcolor)
			{
				// @var object
				$objColor = static::compileColor($objRow->bgcolor);
				// add to Theme
				$objTheme->BackgroundColor = $objColor;
				
				if( empty($objColor->rgb) === false )
				{
					$style = 'background-color:'.$objColor->rgba.';';
					$arrStyles['background-color'] = $style;
				}
			}
			// background image
			if( empty($objRow->bgimage) === false )
			{
				$objFile = FilesModel::findByPath($objRow->bgimage);
				if ( (boolean)Validator::isStringUuid($objRow->bgimage) === true )
				{
					$objFile = FilesModel::findByUuid($objRow->bgimage);
				}
				
				if( $objFile === null && \file_exists(TL_ROOT.'/'.$objRow->bgimage) === true )
				{
					$objFile = \Contao\Dbafs::addResource($objRow->bgimage);
				}

				$style = "background-image:url('".$objFile->path."');";
				$arrStyles['background-image'] = $style;
			}
			// background position
			if($objRow->bgposition)
			{
				#$style = 'background-position:'.$objRow->bgposition.';';
				#$arrStyles['background-position'] = $style;
				$arrClasses[] = $objRow->bgposition;
			}
			// background repeat
			if($objRow->bgrepeat)
			{
				$style = 'background-repeat:'.$objRow->bgrepeat.';';
				$arrStyles['background-repeat'] = $style;
			}
			// background size
			if($objRow->bgsize)
			{
				$style = 'background-size:'.$objRow->bgsize.';';
				$arrStyles['background-size'] = $style;
			}
			// background color class
			$arrClasses[] = $objRow->bgcolor_css;
			
			// fullscreen
			$objTheme->fullscreen = $objRow->fullscreen;
			// just the background styles to this point
			$objTheme->bg_styles = array_filter(array_map('trim',$arrStyles));
		}
		
		// overlay
		if($objRow->overlay)
		{
			$objTheme->overlay = true;
			// @var object
			$objOverlay = new \StdClass;
			// collect overlay classes
			$classes = array( $objRow->ol_bg_color_css, $objRow->ol_position, $objRow->ol_width, $objRow->ol_opacity );
			// overlay height
			if($objRow->ol_height)
			{
				$tmp = \deserialize($objRow->ol_height);
				$objOverlay->height = $tmp['value'].str_replace('pct','%',$tmp['unit']);
				unset($tmp);
			}
			// compile classes
			$objOverlay->classes = implode(' ', array_filter($classes) );
			// add to Theme object
			$objTheme->Overlay = $objOverlay;
			
			unset($classes);		
		}
		
		// custom styles
		if($objRow->styles)
		{
			$arrStyles['custom'] = $objRow->styles;
		}
		
		// clean up
		$arrStyles = array_filter(array_map('trim',$arrStyles));
		
		if(!empty($arrStyles))
		{
			$objTheme->hasStyles = true;
			$objTheme->styles = $arrStyles;
		}
		
		$objTheme->classes = implode(' ', array_unique( array_filter($arrClasses) ) );
		$objTemplate->Theme = $objTheme;
		return $objTemplate;
	}
	
	
	/**
	 * Compile color values coming from contao colorpicker
	 * @param mixed string||array
	 * @return object
	 */
	public static function compileColor($varColor)
	{
		if(is_string($varColor))
		{
			$varColor = \deserialize($varColor);
		}
		
		$tmp = $varColor;
		$color = $tmp[0];
		$rgb = array();
		//-- @see Contao\StyleSheets::convertHexColor
		
		// Try to convert using bitwise operation
		if (strlen($color) == 6)
		{
			$dec = hexdec($color);
			$rgb['red'] = 0xFF & ($dec >> 0x10);
			$rgb['green'] = 0xFF & ($dec >> 0x8);
			$rgb['blue'] = 0xFF & $dec;
		}
		// Shorthand notation
		elseif (strlen($color) == 3)
		{
			$rgb['red'] = hexdec(str_repeat(substr($color, 0, 1), 2));
			$rgb['green'] = hexdec(str_repeat(substr($color, 1, 1), 2));
			$rgb['blue'] = hexdec(str_repeat(substr($color, 2, 1), 2));
		}
		//--
		
		$alpha = '1';
		if($tmp[1])
		{
			$alpha = (int)$tmp[1] / 100;
		}
		
		$objReturn = new \StdClass;
		$objReturn->alpha = $alpha;
		$objReturn->hex = $color;
		$objReturn->rgb = $rgb;
		$objReturn->rgba = 'rgba('.implode(',', $rgb).','.$alpha.')';
	
		return $objReturn;
	}
}