<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2019
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  pct_theme_settings
 * @link  http://contao.org
 */


/**
 * Namespace
 */
namespace PCT\ThemeSettings\Privacy;


/**
 * Class file
 * ModuleOptIn
 */
class ModuleOptIn extends \Contao\Module
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'mod_privacy_default';
	
	/**
	 * The Session and cookie name
	 * @var string
	 */
	protected $strSession = 'user_privacy_settings';
	

	/**
	 * Display a wildcard in the back end
	 *
	 * @return string
	 */
	public function generate()
	{
		if (TL_MODE == 'BE')
		{
			/** @var \BackendTemplate|object $objTemplate */
			$objTemplate = new \Contao\BackendTemplate('be_wildcard');

			$objTemplate->wildcard = '### ' . utf8_strtoupper($GLOBALS['TL_LANG']['FMD']['privacy_in'][0]) . ' ###';
			$objTemplate->title = $this->headline;
			$objTemplate->id = $this->id;
			$objTemplate->link = $this->name;
			$objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id=' . $this->id;

			return $objTemplate->parse();
		}
		
		if( $this->customTpl != $this->strTemplate && empty($this->customTpl) === false )
		{
			$this->strTemplate = $this->customTpl;
		}
		
		return parent::generate();
	}


	/**
	 * Generate the module
	 */
	protected function compile()
	{
		global $objPage;
		
		$this->Template->session_name = $this->strSession;
		$this->Template->privacy = \Contao\Input::cookie($this->strSession);
		
		if(\Contao\Input::post('FORM_SUBMIT') == $this->strSession)
		{
			// privacy settings
			\Contao\Input::setCookie($this->strSession, (int)\Contao\Input::post('privacy') );
			// reload to have the session take effect
			$this->reload();
		}
		
		// labels and descriptions
		$this->Template->label_1 = $this->privacy_lbl_1;
		$this->Template->label_2 = $this->privacy_lbl_2;
		$this->Template->label_3 = $this->privacy_lbl_3;
		$this->Template->description_1 = $this->privacy_txt_1;
		$this->Template->description_2 = $this->privacy_txt_2;
		$this->Template->description_3 = $this->privacy_txt_3;
		// links
		if($this->privacy_url_1)
		{
			$arr = explode('::', str_replace(array('{{','}}'),'',$this->privacy_url_1));
			$this->Template->link_1 = '{{link::'.$arr[1].'}}';
			unset($arr);
		}
		if($this->privacy_url_2)
		{
			$arr = explode('::', str_replace(array('{{','}}'),'',$this->privacy_url_2));
			$this->Template->link_2 = '{{link::'.$arr[1].'}}';
			unset($arr);
		}
		
		// Create an opt-in log file "user_privacy_settings.log" in the system/config folder
		$strFileDir = 'system/config';
		if(\Contao\Environment::get('isAjaxRequest') && \Contao\Input::get($this->strSession) != '')
		{
			$objFile = new \Contao\File($strFileDir.'/user_privacy_settings.log',true);
			$objFile->write( $objFile->getContent() . \Contao\System::parseDate('d-m-Y G:i:s').' '.\Contao\Config::get('timeZone').';IP='.\Contao\Environment::get('ip').';DOMAIN:'.\Contao\Environment::get('base').';PRIVACY='.\Contao\Input::get('user_privacy_settings')."\n" );
			$objFile->close();
		}
		
		// page object to template
		$this->Template->Page = $objPage;
	}
}