<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2019
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  pct_theme_settings
 * @link  http://contao.org
 */


/**
 * Namespace
 */
namespace PCT\ThemeSettings\Privacy;


/**
 * Class file
 * ModuleOptOut
 */
class ModuleOptOut extends \Contao\Module
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'mod_auto_optout';

	/**
	 * The Session and cookie name
	 * @var string
	 */
	protected $strSession = 'user_privacy_settings';
	
	
	/**
	 * Display a wildcard in the back end
	 *
	 * @return string
	 */
	public function generate()
	{
		if (TL_MODE == 'BE')
		{
			/** @var \BackendTemplate|object $objTemplate */
			$objTemplate = new \Contao\BackendTemplate('be_wildcard');

			$objTemplate->wildcard = '### ' . utf8_strtoupper($GLOBALS['TL_LANG']['FMD']['privacy_out'][0]) . ' ###';
			$objTemplate->title = $this->headline;
			$objTemplate->id = $this->id;
			$objTemplate->link = $this->name;
			$objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id=' . $this->id;

			return $objTemplate->parse();
		}
		
		return parent::generate();
	}

	/**
	 * Display a wildcard in the back end
	 *
	 * @return string
	 */
	public function compile()
	{
		global $objPage;
		
		$objJumpTo = null;
		if($this->jumpTo)
		{
			$objJumpTo = \Contao\PageModel::findByPk($this->jumpTo);
		}
		
		$this->Template->session_name = $this->strSession;
		
		// reset the cookie
		\Contao\Input::setCookie($this->strSession, null);
		
		$strRedirect = '';
		
		// Set last page visited
		if ($this->redirectBack)
		{
			$strRedirect = $this->getReferer();
		}
		// Redirect to jumpTo page
		elseif ($objJumpTo !== null && $objJumpTo->id != $objPage->id)
		{
			/** @var \PageModel $objTarget */
			$strRedirect = $objJumpTo->getFrontendUrl();
		}
		
		// check against url
		if( $strRedirect == \Contao\Environment::get('request') )
		{
			$strRedirect = '';
		}
		
		if($strRedirect)
		{
			$this->Template->redirect = $strRedirect;
		}
	}
}