<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2016, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_themer
 */

/**
 * Namespace
 */
namespace PCT\Themer;

/**
 * Imports
 */
use Contao\Database;
use Contao\Dbafs;
use Contao\Input;
use Contao\Controller;
use Contao\System;
use Contao\Validator;
use Contao\StringUtil;

/**
 * Class file
 * Themer
 *
 * Provide various methods to import theme demo templates
 */
class Import extends \PCT\Themer
{	
	/**
	 * Unix timestamp when import started
	 * @var integer
	 */
	protected $intTime = 0;
	
	/**
	 * pid look up array
	 * @var array
	 */
	protected $arrPids = array();
	
	/**
	 * pid look up array
	 * @var array
	 */
	protected $arrNewRecords = array();
	
	/**
	 * Theme record id
	 * @var integer
	 */
	protected $intTheme = 0;
	
	/**
	 * Stores files information
	 * @var array
	 */
	protected $arrTl_files = array();
	
		
	/**
	 * Run the import for a root page id
	 * @param integer
	 * @return boolean	TRUE if the import was successfull
	 */
	public function run($intRoot)
	{
		$objRoot = \Contao\PageModel::findByPk($intRoot);
		if($objRoot->type != 'root' || strlen($objRoot->pct_theme) < 1)
		{
			return false;
		}
		
		// catch theme id from POST on first save
		$this->intTheme = $objRoot->pct_theme_cto;
		if($this->intTheme < 1 && Input::post('pct_theme_cto') > 0)
		{
			$this->intTheme = Input::post('pct_theme_cto');
		}
		
		$this->setRootPage($objRoot);
		
		// includes $arrImport
		include Controller::getTemplate('demo_'.$objRoot->pct_theme,'html5');
		
		if(!isset($arrImport))
		{
			return false;
		}
		
		$this->intRoot = $intRoot;
		$this->intTime = time();
		$objDatabase = Database::getInstance();
		$objSession = System::getContainer()->get('session');
		
		if(is_array($arrImport['sql']['tl_files']))
		{
			$this->arrTl_files = $arrImport['sql']['tl_files'];
			unset($arrImport['sql']['tl_files']);
		}
		
		// rebuild array by tables names as keys
		$arrData = array();
		foreach($arrImport['sql'] as $data)
		{
			if(strlen($data['table']) < 1)
			{
				continue;
			}
			
			$arrData[ $data['table'] ][] = $data;
		}
		
		#$this->arrLookUp = array('tl_content'=>array());
		#foreach($arrData as $table => $records)
		#{
		#	foreach($records as $data)
		#	{
		#		$row = array_combine( array_values($data['fields']),array_values($data['values']) );
		#		$ptable = $row['ptable'];
		#		$id  =$row['id'];
		#		if($ptable)
		#		{
		#			$this->arrLookUp[$table][$ptable][$id] = $row;
		#			continue;
		#		}
		#		$this->arrLookUp[$table][$id] = $row;
		#	}
		#}
		
		$arrVaultData = $arrData['tl_pct_customelement_vault'];
		unset($arrData['tl_pct_customelement_vault']);
		
		#$arrData = array('tl_page'=>$arrData['tl_page'],'tl_article'=>$arrData['tl_article']);
		
		// move tl_content, tl_module on last positions so we have all nessessary records created before
		$arrTables = array_keys($arrData);
		if(in_array('tl_module', $arrTables))
		{
			unset($arrTables[ array_search('tl_module', $arrTables) ]);
			$arrTables[] = 'tl_module';
		}
		if(in_array('tl_content', $arrTables))
		{
			unset($arrTables[ array_search('tl_content', $arrTables) ]);
			$arrTables[] = 'tl_content';
		}
		
		$arrNewRecords = array();
		foreach($arrTables as $strTable)
		{
			$arrNewRecords = $this->mergeData($arrNewRecords,$this->prepareData($strTable,$arrData));
		}
		
		//-- tl_content related
		if(count($arrData['tl_content']) > 0)
		{
			$strTable = 'tl_content';
			foreach($arrData[$strTable] as $data)
			{
				$row = array_combine( array_values($data['fields']),array_values($data['values']));
				
				$oldId = $row['id'];
				$id = $this->arrPids[$strTable][$oldId];
				
				$obj = \Contao\ContentModel::findByPk($id);
				if($obj === null)
				{
					continue;
				}
				
				$row = $obj->row();
						
				// tl_module included 
				if($obj->type == 'module' && $obj->module > 0)
				{
					$table = 'tl_module';
					$field = 'module';
					if($obj->{$field} != $this->arrPids[$table][$row[$field]] && $this->arrPids[$table][$row[$field]] > 0)
					{
						$row[$field] = $this->arrPids[$table][$row[$field]];
						// update the record	
						$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set($row)->execute($obj->id);
					}
				}
				// tl_revolutionslider included
				else if($obj->type == 'revolutionslider' && $obj->revolutionslider > 0)
				{
					$table = 'tl_revolutionslider';
					$field = 'revolutionslider';
					if($obj->{$field} != $this->arrPids[$table][$row[$field]] && $this->arrPids[$table][$row[$field]] > 0)
					{
						$row[$field] = $this->arrPids[$table][$row[$field]];
						// update the record	
						$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set($row)->execute($obj->id);
					}
				}
				// tl_article includes
				else if($obj->type == 'article' && strlen($obj->articleAlias) > 0)
				{
					$table = 'tl_article';
					$field = 'articleAlias';
					if($obj->{$field} != $this->arrPids[$table][$row[$field]] && $this->arrPids[$table][$row[$field]] > 0)
					{
						$row[$field] = $this->arrPids[$table][$row[$field]];
						// update the record	
						$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set($row)->execute($obj->id);
					}
				}
				// tl_content includes
				else if($obj->type == 'alias' && strlen($obj->cteAlias) > 0)
				{
					$table = 'tl_content';
					$field = 'cteAlias';
					if($obj->{$field} != $this->arrPids[$table][$row[$field]] && $this->arrPids[$table][$row[$field]] > 0)
					{
						$row[$field] = $this->arrPids[$table][$row[$field]];
						// update the record	
						$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set($row)->execute($obj->id);
					}
				}
				// tl_form includes
				else if($obj->type == 'form' && strlen($obj->form) > 0)
				{
					$table = 'tl_form';
					$field = 'form';
					if($obj->{$field} != $this->arrPids[$table][$row[$field]] && $this->arrPids[$table][$row[$field]] > 0)
					{
						$row[$field] = $this->arrPids[$table][$row[$field]];
						// update the record	
						$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set($row)->execute($obj->id);
					}
				}
			}
		}
		
		//-- tl_module related
		if(count($arrData['tl_module']) > 0)
		{
			$strTable = 'tl_module';
			foreach($arrData[$strTable] as $data)
			{
				$row = array_combine( array_values($data['fields']),array_values($data['values']));
				
				$oldId = $row['id'];
				$id = $this->arrPids[$strTable][$oldId];
				
				$obj = \Contao\ModuleModel::findByPk($id);
				if($obj === null)
				{
					continue;
				}
				
				$row = $obj->row();
				
				$blnUpdate = false;
				
					
				// tl_news_archive, tl_news, tl_content included
				if( in_array($obj->type, array('newslist','newsreader')) && isset($obj->news_archives))
				{
					$table = 'tl_news_archive';
					$field = 'news_archives';
					$archives = deserialize($obj->{$field});
					
					$tmp = array();
					foreach($archives as $oldArchiveId)
					{
						$tmp[] = $this->arrPids[$table][$oldArchiveId];
					}
					$row[$field] = $tmp;
					unset($tmp);
					
					// update the record	
					$blnUpdate = true;
				}
				// tl_calender, tl_calendar_events, tl_content included
				else if( in_array($obj->type, array('eventlist','eventreader')) && isset($obj->cal_calendar))
				{
					$table = 'tl_calendar';
					$field = 'cal_calendar';
					$archives = deserialize($obj->{$field});
					
					$tmp = array();
					foreach($archives as $oldArchiveId)
					{
						$tmp[] = $this->arrPids[$table][$oldArchiveId];
					}
					$row[$field] = $tmp;
					unset($tmp);
					
					// update the record	
					$blnUpdate = true;
				}
				
				if($blnUpdate)
				{
					// update the record	
					$objDatabase->prepare("UPDATE ".$strTable." %s WHERE id=?")->set($row)->execute($obj->id);
				}
			}
		}
		
		// tl_pct_customelement_vault
		if(is_array($arrVaultData) && count($arrVaultData) > 0)
		{
			$strTable = 'tl_pct_customelement_vault';
			foreach($arrVaultData as $data)
			{
				$row = array_combine( array_values($data['fields']),array_values($data['values']));
				$source = $row['source'];
				if(strlen($source) < 1 || !$objDatabase->tableExists($source))
				{
					continue;
				}
				
				$oldId = $row['id'];
				unset($row['id']);
				
				$intPid = 0;
				if($this->arrPids[$source][$row['pid']] > 0)
				{
					$intPid = $this->arrPids[$source][$row['pid']];
				}
				
				$row['tstamp'] = $this->intTime;
				$row['pid'] = $intPid;
				
				// binary data
				if($row['type'] == 'wizard')
				{
					$arrWizard = deserialize($row['data_blob']);
					if(count($arrWizard) < 1 || !is_array($arrWizard))
					{
						continue;
					}
					
					if(count($arrWizard['values']) < 1)
					{
						continue;
					}
					
					foreach($arrWizard['values'] as $field => $v)
			   		{
			      		if(empty($v))
			      		{
				      		continue;
			      		}
			      		
			      		$v = deserialize($v);
			      		
			      		if(!is_array($v))
				      	{
				      		if(Validator::isStringUuid($v))
							{
								$uuid = $v;
								$bin = null;
								if($this->arrTl_files[$uuid])
								{
									$strFile = $this->arrTl_files[$uuid];
									if(file_exists(TL_ROOT.'/'.$strFile))
									{
										$objFile = Dbafs::addResource($strFile);
										if($objFile)
										{
											$bin = $objFile->uuid;
										}
									}
								}
								else
								{
									$bin = StringUtil::uuidToBin($uuid);
								}
								
								if($bin !== null)
								{
									$arrWizard['values'][$field] = StringUtil::binToUuid($bin);
								}
							}
						}
						// binary in multiSRC fields
				   		else if(in_array($field, $GLOBALS['PCT_THEMER']['multiSRC_fields']) && is_array($v))
				   		{
				   			$arrValue = deserialize($v);
				   			if(is_array($arrValue) && count($arrValue) > 0)
				   			{
				   				$tmp = array();
				   				foreach($arrValue as $v)
				   				{
				   					$bin = null;
									if($this->arrTl_files[$uuid])
									{
										$strFile = $this->arrTl_files[$uuid];
										if(file_exists(TL_ROOT.'/'.$strFile))
										{
											$objFile = Dbafs::addResource($strFile);
											if($objFile)
											{
												$bin = $objFile->uuid;
											}
										}
									}
									else
									{
										$bin = StringUtil::uuidToBin($uuid);
									}
									
									if($bin !== null)
									{
										$tmp[] = StringUtil::binToUuid($bin);
									}
				   				}
				   				
				   				$arrWizard['values'][$field] = serialize($tmp);
				   				unset($tmp);
				   			}
				   		}
					}
					
					$row['data_blob'] = serialize($arrWizard);
				}
				
				
				// INSERT
				$objStatement = $objDatabase->prepare("INSERT INTO ".$strTable." %s")->set($row)->execute();
				$_insertId = $objStatement->__get('insertId');
				
				$this->arrNewRecords[$strTable][] = $_insertId;
				
				// store as new pid
				$this->arrPids[$strTable][$oldId] = $_insertId; 
			}
		}
		
		// get the session data
		$arrSession = $objSession->get('contao_backend');
		$arrSession['PCT_THEMER']['IMPORT'][$this->intRoot] = $this->arrNewRecords;
		$objSession->set('contao_backend',$arrSession);
		
		return true;
	}
	
	
	/**
	 * Prepare the sql array
	 */
	protected function prepareData($strTable, $arrData=array(), array $arrReturn=array())
	{
		$objDatabase = Database::getInstance();
		$arrRecords = $arrData[$strTable];
		
		if(!is_array($arrRecords) || count($arrRecords) < 1 || !$objDatabase->tableExists($strTable))
		{
			return $arrReturn;
		}
		
		// load the datacontainer
		if(!$GLOBALS['TL_DCA'][$strTable]['config'])
		{
			Controller::loadDataContainer($strTable);
		}
		
		// parent table
		$strPtable = '';
		if(strlen($GLOBALS['TL_DCA'][$strTable]['config']['ptable']) > 0)
		{
			$strPtable = $GLOBALS['TL_DCA'][$strTable]['config']['ptable'];
		}
		
		// dynamicPtable
		$blnDynamic = (boolean)$GLOBALS['TL_DCA'][$strTable]['config']['dynamicPtable'];
		if($strTable == 'tl_pct_customelement_vault')
		{
			$blnDynamic = true;
		}
		
		foreach($arrRecords as $i => $data)
		{
			$intPid = 0;
			
			$fields = $data['fields'];
			$values = $data['values'];
			
			// check if field still exists
			foreach($fields as $k => $f)
			{
				if($strTable == 'tl_page' && $f == '_rootId_')
				{
					$intPid = $this->intRoot;	
				}
				
				if(!$objDatabase->fieldExists($f,$strTable))
				{
					unset($fields[$k]);
					unset($values[$k]);
				}
				
				// check if a value is a uuid string
				if(Validator::isStringUuid($values[$k]))
				{
					$uuid = $values[$k];
					
					$bin = null;
					if($this->arrTl_files[$uuid])
					{
						$strFile = $this->arrTl_files[$uuid];
						if(file_exists(TL_ROOT.'/'.$strFile))
						{
							$objFile = Dbafs::addResource($strFile);
							if($objFile)
							{
								$bin = $objFile->uuid;
							}
						}
					}
					else
					{
						$bin = StringUtil::uuidToBin($uuid);
					}
									
					$values[$k] = $bin;
				}
				
				// binary in multiSRC fields
				if(in_array($f, $GLOBALS['PCT_THEMER']['multiSRC_fields']) && !empty($values[$k]))
				{
					$arrValue = deserialize($values[$k]);
					if(is_array($arrValue) && count($arrValue) > 0)
					{
						$tmp = array();
						foreach($arrValue as $uuid)
						{
							$bin = null;
							
							if($this->arrTl_files[$uuid])
							{
								$strFile = $this->arrTl_files[$uuid];
								if(file_exists(TL_ROOT.'/'.$strFile))
								{
									$objFile = Dbafs::addResource($strFile);
									if($objFile)
									{
										$bin = $objFile->uuid;
									}
								}
							}
							else
							{
								$bin = StringUtil::uuidToBin($uuid);
							}
							$tmp[] = $bin;
						}
						
						$values[$k] = serialize($tmp);
						unset($tmp);
						
						
						#$values[$k] = serialize( array_map('StringUtil::uuidToBin', $arrValue) );
					}
				}
			}
			
			// build set array
			$row = array_combine( array_values($fields),array_values($values));
			
			// update timestamp
			if(isset($row['tstamp']))
			{
				$row['tstamp'] = $this->intTime;
			}
			
			// has a ptable field
			if($blnDynamic && strlen($row['ptable']) > 0)
			{
				$strPtable = $row['ptable'];
			}
			// has a source field
			else if($blnDynamic && strlen($row['source']) > 0)
			{
				$strPtable = $row['source'];
			}
			
			// append an info to the title
			if(strlen($row['title']) > 0 && strlen($GLOBALS['PCT_THEMER']['appendTitle']) > 0)
			{
				$row['title'] .= sprintf( $GLOBALS['PCT_THEMER']['appendTitle'],$this->getName(),System::parseDate('d.m.Y H:i',time()) );
			}
			else if(strlen($row['name']) > 0 && strlen($GLOBALS['PCT_THEMER']['appendTitle']) > 0)
			{
				$row['name'] .= sprintf( $GLOBALS['PCT_THEMER']['appendTitle'],$this->getName(),System::parseDate('d.m.Y H:i',time()) );
			}
			
			// remember the old id
			$oldId = $row['id'];
			unset($row['id']);
			
			// find pid
			if(strlen($strPtable) > 0)
			{
				if($row['pid'] > 0 && $this->arrPids[$strPtable][$row['pid']] > 0)
				{
					$intPid = $this->arrPids[$strPtable][$row['pid']];
				}
				else if($strTable == 'tl_module')
				{
					$intPid = $this->intTheme;
				}
			}
			// nested record of same table in MODE 5
			else if(in_array($GLOBALS['TL_DCA'][$strTable]['list']['sorting']['mode'],array(5)) && $row['pid'] > 0 && $this->arrPids[$strTable][$row['pid']] > 0)
			{
				$intPid = $this->arrPids[$strTable][$row['pid']];
			}
			
			// set new pid
			if(isset($row['pid']))
			{
				$row['pid'] = $intPid;
			}
			
			// relink redirects
			foreach($GLOBALS['PCT_THEMER']['singleJumpTo_fields'] as $field)
			{
				$oldJumpTo = $row[$field];
				if($oldJumpTo > 0 && $this->arrPids['tl_page'][$oldJumpTo] > 0 && $oldJumpTo !=  $this->arrPids['tl_page'][$oldJumpTo])
				{
					$row[$field] = $this->arrPids['tl_page'][$oldJumpTo];
				}
			}
			
			// relink multiple page selections
			foreach($GLOBALS['PCT_THEMER']['multiJumpTo_fields'] as $field)
			{
				$pages = deserialize($row[$field]);
				if(empty($pages) || !is_array($pages))
				{
					continue;
				}
				
				$tmp = array();
				foreach($pages as $oldPageId)
				{
					$tmp[] = $this->arrPids['tl_page'][$oldPageId];
				}
				$row[$field] = $tmp;
				unset($tmp);
			}
			
			// INSERT
			$objStatement = $objDatabase->prepare("INSERT INTO ".$strTable." %s")->set($row)->execute();
			$_insertId = $objStatement->__get('insertId');
			
			$arrReturn[$strTable][] = $_insertId;
			
			$this->arrNewRecords[$strTable][] = $_insertId;
			
			// store as new pid
			$this->arrPids[$strTable][$oldId] = $_insertId;
		}
		
		#if(!is_array($GLOBALS['TL_DCA'][$strTable]['config']['ctable']) || count($GLOBALS['TL_DCA'][$strTable]['config']['ctable']) < 1 )
		#{
		#	return $arrReturn;
		#}
			
		#foreach($GLOBALS['TL_DCA'][$strTable]['config']['ctable'] as $ctable)
		#{
		#	// continue if table does not exist or we have no records to insert
		#	if(!$objDatabase->tableExists($ctable))
		#	{
		#		continue;
		#	}
		#	
		#	if(!is_array($arrReturn[$ctable]))
		#	{
		#		$arrReturn[$ctable] = array();
		#	}
		#	
		#	if(!$GLOBALS['TL_DCA'][$ctable]['config'])
		#	{
		#		Controller::loadDataContainer($ctable);
		#	}
		#	
		#	$arrReturn = array_merge($arrReturn,$this->prepareData($ctable,$arrData,$arrReturn));
		#}
		
		return $arrReturn;
	}
	
	
}